SystemOrganization addCategory: #'Pier-Seaside-Core'!
SystemOrganization addCategory: #'Pier-Seaside-Deprecated'!
SystemOrganization addCategory: #'Pier-Seaside-Libraries'!
SystemOrganization addCategory: #'Pier-Seaside-Magritte'!
SystemOrganization addCategory: #'Pier-Seaside-Model'!
SystemOrganization addCategory: #'Pier-Seaside-View'!
SystemOrganization addCategory: #'Pier-Seaside-Visitors'!
SystemOrganization addCategory: #'Pier-Seaside-Widgets'!
SystemOrganization addCategory: #'Pier-Seaside-Widgets-Settings'!

!PRLink methodsFor: '*pier-seaside' stamp: 'lr 1/30/2009 20:24'!
hasCounter
	^ self hasParameter: 'counter'! !

!PRLink methodsFor: '*pier-seaside' stamp: 'lr 1/30/2009 20:31'!
incrementCounter
	"Increment the counter by one, if defined as a property of the receiver."

	| value |
	value := self parameterAt: 'counter' ifAbsent: [ ^ self ].
	value := value isNil ifTrue: [ 0 ] ifFalse: [ value asNumber ].
	^ self parameterAt: 'counter' put: value + 1! !

MATextInputComponent subclass: #PRStructureComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Magritte'!

!PRStructureComponent methodsFor: 'actions' stamp: 'lr 11/2/2007 16:15'!
choose
	| tree |
	tree := WATree root: self context root path: self context structure parents.
	tree
		labelBlock: [ :structure | structure title ];
		childrenBlock: [ :structure |
			(self context enumeratorFor: structure)
				contents ];
		canSelectBlock: [ :structure | 
			self description isSatisfiedBy: structure ].
	self chooser: tree! !

!PRStructureComponent methodsFor: 'rendering' stamp: 'lr 3/27/2006 11:32'!
renderEditorOn: html
	super renderEditorOn: html.
	html submitButton on: #choose of: self! !

!PRStructureComponent methodsFor: 'rendering' stamp: 'lr 11/5/2007 22:10'!
renderViewerOn: html
	self value isNil
		ifTrue: [ ^ html render: self string ].
	html anchor
		goto: (self context structure: self value);
		with: self string! !

!PRWordDiffAlgorithm methodsFor: '*pier-seaside-rendering' stamp: 'lr 9/28/2009 19:58'!
renderOn: html
	self patchsetDo: [ :operation :object :index |
		operation = #-
			ifTrue: [ html deleted: [ self renderText: object spaces: (sourceSpaces at: index) on: html ] ]
			ifFalse: [ 
				operation = #+
					ifTrue: [ html inserted: [ self renderText: object spaces: (targetSpaces at: index) on: html ] ]
					ifFalse: [ self renderText: object spaces: (targetSpaces at: index) on: html ] ] ]! !

!PRWordDiffAlgorithm methodsFor: '*pier-seaside-rendering' stamp: 'lr 9/28/2009 15:10'!
renderText: aString spaces: aSpacesString on: html
	| stream char |
	html text: aString.
	stream := aSpacesString readStream.
	[ stream atEnd ] whileFalse: [ 
		char := stream next.
		char = Character cr
			ifTrue: [ html break ]
			ifFalse: [ html text: char ] ]! !

!WAPresenter methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
context
	^ PRCurrentContext value! !

!WAPresenter methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
context: aContext
	PRCurrentContext value: aContext! !

!WAPresenter methodsFor: '*pier-seaside-events' stamp: 'lr 3/9/2006 11:30'!
onChangeContext: aContext
	"Inform components about a new context that will be used from now on."! !

MAComponent subclass: #PRPierFrame
	instanceVariableNames: 'context children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Core'!

!PRPierFrame class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
canBeRoot
	^ true! !

!PRPierFrame class methodsFor: 'private' stamp: 'lr 6/11/2009 09:27'!
classFromRequest: aRequest name: aString base: aBaseClass
	| value |
	value := aRequest
		at: aString
		ifAbsent: [ ^ nil ].
	^ aBaseClass withAllConcreteClasses
		detect: [ :each | 
			each isAbstract not
				and: [ each name = value 
				or: [ each label = value ] ] ]
		ifNone: [ nil ]! !

!PRPierFrame class methodsFor: 'registration' stamp: 'NickAger 1/11/2010 13:57'!
configureApplication: application
	application configuration
		addParent: PRPierConfiguration instance.
	application 
		addLibrary: PRBlueprintLibrary.
	Smalltalk at: #PTDeploymentLibrary ifPresent: [:class |
		application addLibrary: class].
	Smalltalk at: #SUDeploymentLibrary ifPresent: [ :class | 
		application 
			addLibrary: class; 
			addLibrary: PRJavaScriptSupport ]! !

!PRPierFrame class methodsFor: 'registration' stamp: 'lr 12/19/2009 16:35'!
description
	^ 'Pier'! !

!PRPierFrame class methodsFor: 'examples' stamp: 'lr 8/24/2008 08:18'!
exampleDocumentation
	"Create a (static) Pier instance from all the comments in the source code of pier, a sort of poor mans documentation. It might take a while to get all the classes traversed and to parse the source."

	^ self on: (PRContext 
		kernel: (PRKernel new
			name: 'Documentation';
			root: ((PRPage named: 'documentation')
				addChild: (self pageForClass: PRObject);
				addChild: (self pageForClass: MAObject);
				yourself);
			yourself))! !

!PRPierFrame class methodsFor: 'examples' stamp: 'lr 1/5/2010 18:05'!
exampleEmpty
	"Pier can be easily used as a component within any Seaside application, this is just an example of a dumb wiki. To see this example browse to /seaside/examples/examplebrowser and select the receiver class."

	^ self on: (PRContext kernel: (PRKernel new
		name: 'Pier';
		root: (PRPage new
			title: 'Pier Example';
			contents: 'Pier can be easily used as a component within any Seaside application (maybe you should adapt the style-sheets), this is just an example of a dumb and non-persistent wiki.';
			yourself)))! !

!PRPierFrame class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRPierFrame class methodsFor: 'testing' stamp: 'lr 8/29/2008 11:36'!
isDeployed
	^ true! !

!PRPierFrame class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:30'!
on: aContext
	"This contructor creates a new SmallWiki frame that can be used within any Seaside application. It doesn't require a special session class, since all the required state is hold in the context of the wiki itself."

	^ self new
		context: aContext;
		yourself! !

!PRPierFrame class methodsFor: 'examples' stamp: 'jf 9/30/2009 00:19'!
pageForClass: aClass 
	| page |
	page := (PRPage named: aClass name)
		contents: (String streamContents: 
				[ :stream | 
				stream
					nextPutAll: '!!Class definition';
					nextPut: Character cr.
				aClass definition linesDo: 
					[ :each | 
					(GRPlatform current magritteClassNamed: #SPHighlightedCode) isNil 
						ifTrue: [ stream nextPut: $= ]
						ifFalse: [ stream nextPutAll: '==' ].
					stream
						nextPutAll: each;
						nextPut: Character cr ].
				aClass hasComment ifTrue: 
					[ stream
						nextPutAll: '!!Class comment';
						nextPut: Character cr.
					stream
						nextPutAll: aClass comment;
						nextPut: Character cr ].
				aClass selectorsAndMethodsDo: 
					[ :selector :method | 
					(aClass firstCommentAt: selector) ifNotEmptyDo: 
						[ :comment | 
						stream
							nextPutAll: '!!!!';
							nextPutAll: selector;
							nextPut: Character cr.
						stream
							nextPutAll: comment;
							nextPut: Character cr;
							nextPut: Character cr ] ] ]);
		yourself.
	aClass subclassesDo: [ :each | page addChild: (self pageForClass: each) ].
	^ page! !

!PRPierFrame class methodsFor: 'registration' stamp: 'lr 1/11/2010 23:33'!
registerAsApplication: aString kernel: aKernel
	"self registerAsApplication: 'pier' kernel: (PRKernel named: 'pier')"

	| app |
	app := WAAdmin register: self asApplicationAt: aString.
	self configureApplication: app.
	app preferenceAt: #kernel put: aKernel.
	^ app! !

!PRPierFrame methodsFor: 'visiting' stamp: 'jf 9/14/2009 11:44'!
acceptDecorated: aPainterVisitor
	"Catch any actions done with visitors and make sure they have a context.
	This is hacky because it is really only needed for the initial render when
	the PRContextFilter is not in the dynamic context. This might be fixable.
	Note that this is not used with callback processing; see PRContextFilter."
	
	self withContextDo: [super acceptDecorated: aPainterVisitor]! !

!PRPierFrame methodsFor: 'private' stamp: 'lr 2/14/2008 13:24'!
buildChildren
	^ Set new
		addAll: (self componentsOf: self environment);
		addAll: (self componentsOf: self structure);
		yourself! !

!PRPierFrame methodsFor: 'accessing-dynamic' stamp: 'lr 9/30/2007 18:00'!
children
	^ children ifNil: [ children := self buildChildren ]! !

!PRPierFrame methodsFor: 'private' stamp: 'lr 2/14/2008 09:35'!
componentsOf: aStructure
	^ aStructure componentsIn: self context! !

!PRPierFrame methodsFor: 'accessing' stamp: 'lr 12/19/2009 16:17'!
context
	"Answer the current context of Pier, initialize with a default context if nil."

	^ context ifNil: [ context := PRContext kernel: self kernel ]! !

!PRPierFrame methodsFor: 'accessing' stamp: 'lr 9/4/2008 12:35'!
context: aContext
	aContext isValid
		ifFalse: [ ^ self forbidden: aContext ].
	context := aContext.
	children := nil.
	self update! !

!PRPierFrame methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:30'!
environment
	^ self context structure environment! !

!PRPierFrame methodsFor: 'processing-errors' stamp: 'jf 9/6/2009 20:46'!
forbidden: aContext
	"This method is called whenever aContext is forbidden to be accessed. If everything fails, we show a plain error notification."
	
	| error |
	error := aContext
		structure: aContext structure
		view: PRForbiddenView.
	[ error structure isRoot or: [ error isValid ] ] whileFalse: [ 
		error := error
			structure: error structure parent
			view: PRForbiddenView ].
	error command viewComponent 
		forbidden: aContext copy.
	error isValid
		ifTrue: [ self context: error ]
		ifFalse: [
			self requestContext responseGenerator
				forbidden;
				respond ]! !

!PRPierFrame methodsFor: 'processing' stamp: 'lr 1/11/2010 23:44'!
initialRequest: aRequest
	| structure following |
	super initialRequest: aRequest.
	self session 
		addFilter: (PRContextFilter on: self).
	following := self context
		structure: (structure := self 
			parseStructure: aRequest 
			ifAbsent: [ ^ self notFound ])
		command: (self
			parseCommand: aRequest
			structure: structure).
	following command
		initialRequest: aRequest.
	self context: following! !

!PRPierFrame methodsFor: 'accessing-dynamic' stamp: 'lr 1/5/2010 18:03'!
kernel
	^ self application preferenceAt: #kernel! !

!PRPierFrame methodsFor: 'processing-errors' stamp: 'lr 1/5/2010 18:03'!
notFound
	"This method is called whenever the current request cannot be resolved to a structure."
	
	self context: (self context
		structure: self context root
		view: PRNotFoundView)! !

!PRPierFrame methodsFor: 'processing' stamp: 'lr 12/19/2009 16:37'!
parseCommand: aRequest structure: aStructure
	| commandClass |
	commandClass := self class 
		classFromRequest: aRequest 
		name: 'command'
		base: PRCommand.
	(commandClass isNil or: [ commandClass isAbstract ])
		ifTrue: [ commandClass := aStructure defaultCommandClass ].
	^ commandClass new! !

!PRPierFrame methodsFor: 'processing' stamp: 'jf 1/8/2010 21:49'!
parseStructure: aRequest ifAbsent: aBlock
	| path consumer pathString |
	self flag: #todo. "This should be improveable using #upToEnd once it is added"
	
	path := OrderedCollection new.
	consumer := self requestContext consumer.
	[ consumer atEnd ] whileFalse:
		[ path add: consumer next ].
	pathString := path
		inject: String new
		into: [ :result :each | result , '/' , each ].
	^ self context root
		lookupPath: pathString
		ifAbsent: aBlock! !

!PRPierFrame methodsFor: 'rendering' stamp: 'lr 4/8/2009 20:16'!
renderContentOn: html
	html rendererInstance
		start: self environment
		in: self on: html! !

!PRPierFrame methodsFor: 'private' stamp: 'jf 9/9/2009 15:00'!
rendererForRoot: aHtmlRoot
	^ self rendererClass context: aHtmlRoot context! !

!PRPierFrame methodsFor: 'accessing-dynamic' stamp: 'lr 7/13/2007 09:55'!
states
	^ Array with: self! !

!PRPierFrame methodsFor: 'accessing-dynamic' stamp: 'lr 2/3/2007 18:28'!
structure
	^ self context structure! !

!PRPierFrame methodsFor: 'updating' stamp: 'lr 9/3/2008 14:13'!
update
	self visiblePresentersDo: [ :each | each onChangeContext: self context ]! !

!PRPierFrame methodsFor: 'updating' stamp: 'lr 4/1/2009 22:41'!
updateBody: aHtmlRoot
	aHtmlRoot beXhtml10Strict.
	aHtmlRoot bodyAttributes addClass: self structure cssClass.
	aHtmlRoot meta name: 'generator'; content: 'Pier - Magritte - Seaside'! !

!PRPierFrame methodsFor: 'updating-extras' stamp: 'lr 2/14/2009 18:14'!
updateCanonical: aHtmlRoot
	"Defines the canonical URL, the URL for the current page that should be used when search engines index or when people bookmark the current page."

	aHtmlRoot link
		relationship: 'canonical';
		url: self context url! !

!PRPierFrame methodsFor: 'updating' stamp: 'lr 4/1/2009 22:41'!
updateDtd: aHtmlRoot
	aHtmlRoot beXhtml10Strict! !

!PRPierFrame methodsFor: 'updating-extras' stamp: 'jf 11/4/2009 10:25'!
updateIcon: aHtmlRoot
	| structure |
	structure := self structure icon ifNil: [ ^ self ].
	structure file isNil ifTrue: [ ^ self ].
	aHtmlRoot link
		beShortcutIcon;
		type: structure file mimetype;
		url: structure file url! !

!PRPierFrame methodsFor: 'updating-extras' stamp: 'lr 3/10/2009 08:26'!
updateNavigation: aHtmlRoot
	| siblings index |
	siblings := self context siblings.
	index := siblings indexOf: self structure.
	self structure isRoot ifFalse: [
		self 
			updateRelationship: 'top'
			with: self structure root
			on: aHtmlRoot ].
	self
		updateRelationship: 'up'
		with: self structure parent
		on: aHtmlRoot.
	self 
		updateRelationship: 'previous'
		with: (siblings at: index - 1 ifAbsent: [ nil ])
		on: aHtmlRoot.
	self 
		updateRelationship: 'next'
		with: (siblings at: index + 1 ifAbsent: [ nil ])
		on: aHtmlRoot! !

!PRPierFrame methodsFor: 'updating-extras' stamp: 'lr 3/26/2009 22:06'!
updateRelationship: aString with: aStructure on: aHtmlRoot
	| target renderer |
	aStructure isNil ifTrue: [ ^ self ].
	target := self context structure: aStructure.
	target isValid ifFalse: [ ^ self ].
	renderer := self rendererForRoot: aHtmlRoot.
	aHtmlRoot link
		relationship: aString; 
		url: (target urlOn: renderer)! !

!PRPierFrame methodsFor: 'updating' stamp: 'kph 3/26/2009 18:33'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	self
		updateDtd: aHtmlRoot;
		updateBody: aHtmlRoot;
		updateStyleSheets: aHtmlRoot;
		updateNavigation: aHtmlRoot;
		updateCanonical: aHtmlRoot;
		updateIcon: aHtmlRoot! !

!PRPierFrame methodsFor: 'updating-extras' stamp: 'jf 11/4/2009 10:25'!
updateStyleSheets: aHtmlRoot
	self structure styleSheets do: [ :structure |
		structure file isEmpty ifFalse: [
			aHtmlRoot stylesheet 
				url: structure file url ] ]! !

!PRPierFrame methodsFor: 'updating' stamp: 'lr 8/15/2007 14:18'!
updateUrl: anUrl
	super updateUrl: anUrl.
	anUrl adaptToContext: self context! !

!PRPierFrame methodsFor: 'private' stamp: 'lr 9/3/2008 14:24'!
withContextDo: aBlock
	^ PRCurrentContext use: self during: aBlock! !

MAComponent subclass: #PRViewComponent
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

PRViewComponent subclass: #PRDefaultView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRDefaultView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRDefaultView class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'View'! !

!PRDefaultView methodsFor: 'testing' stamp: 'lr 7/13/2009 20:55'!
isRestful
	^ true! !

!PRDefaultView methodsFor: 'accessing' stamp: 'lr 1/29/2008 09:06'!
label
	^ self context structure title! !

!PRDefaultView methodsFor: 'rendering' stamp: 'lr 5/19/2009 11:22'!
render: aStructure on: html
	html rendererInstance withinContentDo: [ 
		html rendererInstance 
			continue: aStructure 
			in: self on: html ]! !

!PRDefaultView methodsFor: 'rendering' stamp: 'lr 1/17/2007 23:29'!
renderContentOn: html
	self render: self context structure on: html! !

!PRDefaultView methodsFor: 'accessing' stamp: 'lr 2/20/2007 15:08'!
robots
	^ 'index,follow'! !

!PRDefaultView methodsFor: 'updating' stamp: 'jf 9/6/2009 21:56'!
updateRoot: aHtmlRoot
	"Add universal edit button: http://universaleditbutton.org/."

	| editContext |
	super updateRoot: aHtmlRoot.
	editContext := self context command: self context structure
		editCommandClass new.
	editContext isValid ifTrue: [
		aHtmlRoot link
			beAlternate;
			type: 'application/x-wiki';
			title: editContext command label;
			url: (editContext urlOn: (self rendererClass
				context: aHtmlRoot context)) ]! !

PRViewComponent subclass: #PRErrorView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRErrorView class methodsFor: 'testing' stamp: 'lr 3/17/2009 17:28'!
isListed
	^ false! !

!PRErrorView class methodsFor: 'testing' stamp: 'lr 9/4/2008 12:05'!
isValidIn: aContext
	^ true! !

!PRErrorView class methodsFor: 'accessing' stamp: 'lr 9/4/2008 11:28'!
label
	^ 'Error'! !

!PRErrorView methodsFor: 'rendering' stamp: 'jf 9/29/2009 10:33'!
renderContentOn: html
	self requestContext response status: self status.
	self renderErrorOn: html.
	self renderSolutionsOn: html! !

!PRErrorView methodsFor: 'rendering' stamp: 'lr 9/4/2008 12:31'!
renderErrorOn: html! !

!PRErrorView methodsFor: 'rendering-options' stamp: 'jf 9/14/2009 15:59'!
renderOptionExternalOn: html
	<option: 400>
	
	| referer |
	referer := self requestContext request referer.
	referer isNil ifFalse: [
		html listItem: [
			html text: 'If you clicked a link from an external page '.
			html anchor
				url: referer;
				with: referer.
			html text: ', please inform the webmaster of this site of his mistake.' ] ]! !

!PRErrorView methodsFor: 'rendering-options' stamp: 'lr 9/4/2008 12:18'!
renderOptionLinkOn: html
	<option: 100>
	
	html listItem: 'Click on a link on this site.'! !

!PRErrorView methodsFor: 'rendering-options' stamp: 'lr 9/4/2008 11:27'!
renderOptionRootOn: html
	<option: 200>
	
	| rootContext |
	rootContext := self context structure: self context root.
	rootContext isValid ifTrue: [
		html listItem: [
			html text: 'Jump to the '.
			html anchor
				goto: rootContext;
				with: rootContext structure title.
			html text: ' homepage.' ] ]! !

!PRErrorView methodsFor: 'rendering-options' stamp: 'lr 9/4/2008 11:27'!
renderOptionSpellingOn: html
	<option: 300>
	
	html listItem: 'If you typed the page address, make sure that it is spelled correctly.'! !

!PRErrorView methodsFor: 'rendering' stamp: 'lr 4/6/2009 23:58'!
renderOptionsOn: html
	(Pragma allNamed: #option: from: self class to: PRErrorView sortedUsing: [ :a :b | a arguments first < b arguments first])
		do: [ :each | self perform: each selector with: html ]! !

!PRErrorView methodsFor: 'rendering' stamp: 'lr 9/4/2008 12:18'!
renderSolutionsOn: html
	html paragraph: 'You might want to try one of these options:'.
	html unorderedList: [ self renderOptionsOn: html ]! !

!PRErrorView methodsFor: 'accessing' stamp: 'lr 4/4/2009 17:15'!
status
	^ 200! !

PRErrorView subclass: #PRForbiddenView
	instanceVariableNames: 'forbidden'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRForbiddenView class methodsFor: 'testing' stamp: 'lr 9/4/2008 11:49'!
isAbstract
	^ false! !

!PRForbiddenView class methodsFor: 'accessing' stamp: 'lr 9/4/2008 11:49'!
label
	^ 'Forbidden'! !

!PRForbiddenView methodsFor: 'accessing' stamp: 'lr 9/4/2008 12:38'!
forbidden: aContext
	forbidden := aContext! !

!PRForbiddenView methodsFor: 'rendering' stamp: 'lr 9/4/2008 14:19'!
renderErrorOn: html
	html paragraph: [ 
		html text: 'You are not allowed to '.
		html text: (forbidden isNil
			ifTrue: [ 'view' ]
			ifFalse: [ forbidden command class label asLowercase ]).
		html text: ' this page.' ]! !

!PRForbiddenView methodsFor: 'accessing' stamp: 'lr 4/4/2009 17:16'!
status
	^ 403! !

PRErrorView subclass: #PRNotFoundView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRNotFoundView class methodsFor: 'testing' stamp: 'lr 9/4/2008 11:49'!
isAbstract
	^ false! !

!PRNotFoundView class methodsFor: 'accessing' stamp: 'lr 9/4/2008 11:49'!
label
	^ 'Not Found'! !

!PRNotFoundView methodsFor: 'rendering' stamp: 'jf 9/14/2009 15:59'!
renderErrorOn: html
	html paragraph: [ 
		html 
			text: 'The requested object '; 
			code: self requestContext request url;
			text: ' was not found on this server.' ]! !

!PRNotFoundView methodsFor: 'rendering-options' stamp: 'lr 5/27/2009 16:03'!
renderSearchOn: html
	<option: 500>
	
	| widgets |
	widgets := self context structure environment embeddedComponents
		select: [ :each | each target componentClass includesBehavior: PRSearchWidget ].
	widgets isEmpty ifTrue: [ ^ self ].
	html listItem: 'Use the search on this page to find what you are looking for.'! !

!PRNotFoundView methodsFor: 'accessing' stamp: 'lr 4/4/2009 17:16'!
status
	^ 404! !

PRViewComponent subclass: #PRHistoryView
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

PRHistoryView subclass: #PRChangesView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRChangesView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRChangesView class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Changes'! !

!PRChangesView methodsFor: 'private' stamp: 'lr 11/20/2007 16:24'!
buildReport
	^ super buildReport
		addColumn: (MACommandColumn new
			addCommand: [ :value | self open: value copy ] text: 'open';
			yourself);
		yourself! !

!PRChangesView methodsFor: 'private' stamp: 'lr 11/20/2007 16:16'!
isVisible: aContext
	^ aContext command isEdit and: [ super isVisible: aContext ]! !

PRHistoryView subclass: #PRDiffView
	instanceVariableNames: 'contexts sourceContext targetContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRDiffView class methodsFor: 'testing' stamp: 'lr 9/28/2009 10:07'!
isAbstract
	^ false! !

!PRDiffView class methodsFor: 'accessing' stamp: 'lr 9/28/2009 10:07'!
label
	^ 'Diff'! !

!PRDiffView methodsFor: 'private' stamp: 'lr 9/28/2009 10:23'!
buildList
	"Answer a list of all edit contexts in the receiver that are visible."

	^ ((self context kernel persistency historyFor: self context structure) 
		select: [ :each | self isVisible: each ])
		reversed! !

!PRDiffView methodsFor: 'accessing' stamp: 'lr 9/28/2009 10:05'!
children
	^ #()! !

!PRDiffView methodsFor: 'accessing' stamp: 'lr 9/28/2009 10:10'!
contexts
	^ contexts ifNil: [ contexts := self buildList ]! !

!PRDiffView methodsFor: 'testing' stamp: 'lr 9/28/2009 11:58'!
isVisible: aContext
	^ aContext command isEdit and: [ super isVisible: aContext ]! !

!PRDiffView methodsFor: 'private' stamp: 'lr 9/28/2009 15:22'!
labelFor: aContext
	^ String streamContents: [ :stream |
		stream print: aContext command date; space.
		stream print: aContext command time; space.
		(aContext respondsTo: #user)
			ifTrue: [ stream nextPut: $(; nextPutAll: aContext user name; nextPut: $) ] ]! !

!PRDiffView methodsFor: 'rendering' stamp: 'lr 9/28/2009 10:07'!
renderContentOn: html
	self renderSelectorOn: html.
	self renderDiffOn: html! !

!PRDiffView methodsFor: 'rendering' stamp: 'lr 9/28/2009 19:41'!
renderDiffOn: html
	| someContext sourceString targetString |
	(sourceContext isNil and: [ targetContext isNil ])
		ifTrue: [ ^ self ].
	someContext := sourceContext isNil
		ifTrue: [ targetContext ]
		ifFalse: [ sourceContext ].
	html table: [
		someContext command description do: [ :desc |
			html tableRow: [
				html tableHeading: desc label.
				html tableData: [
					sourceString := sourceContext isNil
						ifFalse: [ desc toString: (sourceContext command readUsing: desc) ].
					targetString := targetContext isNil
						ifFalse: [ desc toString: (targetContext command readUsing: desc) ].
					html render: (PRWordDiffAlgorithm from: sourceString to: targetString) ] ] ] ]! !

!PRDiffView methodsFor: 'rendering' stamp: 'lr 9/28/2009 15:16'!
renderSelectorOn: html
	html form: [
		html table: [
			html tableRow: [
				html tableHeading: 'Source'.
				html tableHeading: 'Target'.
				html tableHeading ].
			html tableRow: [
				html tableData: [
					html select
						list: self contexts;
						selected: self sourceContext;
						labels: [ :value | self labelFor: value ];
						callback: [ :value | sourceContext := value ] ].
				html tableData: [
					html select
						list: self contexts;
						selected: self targetContext;
						labels: [ :value | self labelFor: value ];
						callback: [ :value | targetContext := value ] ].
				html tableData: [
					html submitButton: 'update' ] ] ] ]! !

!PRDiffView methodsFor: 'accessing' stamp: 'lr 9/28/2009 10:10'!
sourceContext
	^ sourceContext ifNil: [ sourceContext := self contexts at: 2 ifAbsent: [ nil ] ]! !

!PRDiffView methodsFor: 'accessing' stamp: 'lr 9/28/2009 10:10'!
targetContext
	^ targetContext ifNil: [ targetContext := self contexts at: 1 ifAbsent: [ nil ] ]! !

!PRHistoryView methodsFor: 'actions' stamp: 'lr 11/20/2007 16:20'!
apply: aContext
	aContext command execute! !

!PRHistoryView methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
buildDescription
	^ PRContext description! !

!PRHistoryView methodsFor: 'private' stamp: 'lr 9/28/2009 10:05'!
buildList
	^ Array streamContents: [ :stream |
		self context kernel persistency reverseDo: [ :each | 
			(self isVisible: each)
				ifTrue: [ stream nextPut: each ] ] ]! !

!PRHistoryView methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
buildReport
	^ (MAReport rows: self buildList description: self buildDescription)
		sortEnabled: false;
		batchSize: 20;
		yourself! !

!PRHistoryView methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
children
	^ Array with: self report! !

!PRHistoryView methodsFor: 'testing' stamp: 'lr 2/2/2010 14:55'!
isVisible: aContext
	^ (aContext structure parents includes: self context structure)
		and: [ (aContext isValidCommand: aContext structure viewCommandClass)
		and: [ aContext structure parents allSatisfy: [ :each | (self context structure: each) isValid ] ] ]! !

!PRHistoryView methodsFor: 'actions' stamp: 'lr 3/20/2008 11:32'!
open: aContext
	(self context isValidCommand: aContext command class)
		ifFalse: [ self inform: 'Unable to open ' , aContext command label printString , ' in the current context.' ]
		ifTrue: [ self context: aContext ]! !

!PRHistoryView methodsFor: 'rendering' stamp: 'lr 3/9/2006 11:30'!
renderContentOn: html
	html render: self report! !

!PRHistoryView methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
report
	^ report ifNil: [ report := self buildReport ]! !

PRHistoryView subclass: #PRLogView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRLogView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRLogView class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Log'! !

PRViewComponent subclass: #PRRawView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

PRRawView subclass: #PRDownloadView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRDownloadView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRDownloadView class methodsFor: 'testing' stamp: 'lr 1/29/2007 17:10'!
isValidIn: aContext
	^ (super isValidIn: aContext) and: [ aContext structure isFile ]! !

!PRDownloadView class methodsFor: 'converting' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Download'! !

!PRDownloadView methodsFor: 'accessing' stamp: 'lr 1/20/2007 18:47'!
file
	^ self context structure file! !

!PRDownloadView methodsFor: 'responding' stamp: 'jf 11/4/2009 11:38'!
respondUsing: aResponse
	super respondUsing: aResponse.
	aResponse
		redirectTo: self file url;
		cacheForever;
		movedPermanently! !

PRRawView subclass: #PRPlainView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRPlainView class methodsFor: 'testing' stamp: 'lr 2/6/2007 18:58'!
isAbstract
	^ false! !

!PRPlainView class methodsFor: 'testing' stamp: 'lr 2/6/2007 18:58'!
isValidIn: aContext
	^ (super isValidIn: aContext) and: [ aContext structure hasDocument ]! !

!PRPlainView class methodsFor: 'converting' stamp: 'lr 2/6/2007 18:58'!
label
	^ 'Text'! !

!PRPlainView methodsFor: 'responding' stamp: 'jf 11/4/2009 11:38'!
respondUsing: aResponse
	super respondUsing: aResponse.
	aResponse contentType: 'text/plain'.
	PRTextWriter
		write: self context structure document
		to: aResponse stream! !

!PRRawView methodsFor: 'testing' stamp: 'jf 11/4/2009 11:05'!
isFullResponse
	^ true! !

!PRRawView methodsFor: 'testing' stamp: 'lr 7/13/2009 20:55'!
isRestful
	^ true! !

!PRRawView methodsFor: 'responding' stamp: 'jf 11/4/2009 11:54'!
respondUsing: aResponse
	aResponse cacheForever! !

PRRawView subclass: #PRWikiView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRWikiView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRWikiView class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isValidIn: aContext
	^ (super isValidIn: aContext) and: [ aContext structure hasDocument ]! !

!PRWikiView class methodsFor: 'converting' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Wiki'! !

!PRWikiView methodsFor: 'responding' stamp: 'jf 11/4/2009 11:38'!
respondUsing: aResponse
	| structure |
	super respondUsing: aResponse.
	structure := self context structure.
	aResponse contentType: 'text/plain'.
	aResponse stream nextPutAll:
		(structure class descriptionDocument 
			toString: structure document)! !

PRViewComponent subclass: #PRSearchView
	instanceVariableNames: 'query items hideForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-View'!

!PRSearchView class methodsFor: 'testing' stamp: 'lr 9/12/2009 11:40'!
isAbstract
	^ false! !

!PRSearchView class methodsFor: 'accessing' stamp: 'lr 9/12/2009 11:40'!
label
	^ 'Search'! !

!PRSearchView methodsFor: 'testing' stamp: 'lr 9/12/2009 16:47'!
hasSearchForm
	^ hideForm not! !

!PRSearchView methodsFor: 'testing' stamp: 'lr 9/12/2009 16:52'!
hasSearchResults
	^ self hasValidQuery and: [ self items notEmpty ]! !

!PRSearchView methodsFor: 'testing' stamp: 'lr 9/12/2009 16:52'!
hasValidQuery
	^ self query notNil and: [ self query notEmpty ]! !

!PRSearchView methodsFor: 'actions' stamp: 'lr 9/12/2009 13:05'!
hideForm
	hideForm := true! !

!PRSearchView methodsFor: 'initialization' stamp: 'lr 9/12/2009 11:36'!
initialRequest: aRequest
	super initialRequest: aRequest.
	aRequest fields 
		at: self queryFieldName
		ifPresent: [ :value | self query: value ]! !

!PRSearchView methodsFor: 'initialization' stamp: 'lr 9/12/2009 13:05'!
initialize
	super initialize.
	hideForm := false.
	items := OrderedCollection new! !

!PRSearchView methodsFor: 'testing' stamp: 'lr 9/12/2009 11:34'!
isRestful
	^ true! !

!PRSearchView methodsFor: 'accessing-readonly' stamp: 'lr 9/12/2009 16:52'!
items
	^ items! !

!PRSearchView methodsFor: 'accessing' stamp: 'lr 9/12/2009 16:52'!
query
	^ query! !

!PRSearchView methodsFor: 'accessing' stamp: 'lr 9/12/2009 11:46'!
query: aString
	query := aString.
	self refresh! !

!PRSearchView methodsFor: 'configuration' stamp: 'lr 9/12/2009 11:36'!
queryFieldName
	^ 'query'! !

!PRSearchView methodsFor: 'actions' stamp: 'lr 9/12/2009 16:53'!
refresh
	| searcher |
	items := OrderedCollection new.
	self hasValidQuery ifFalse: [ ^ self ].
	searcher := self context root fullTextSearch.
	(self query findTokens: Character space) do: [ :token | 
		(token notEmpty and: [ token first = $+ ]) 
			ifTrue: [ searcher acceptToken: token allButFirst ]
			ifFalse: [
				(token notEmpty and: [ token first = $- ]) 
					ifTrue: [ searcher rejectToken: token allButFirst ]
					ifFalse: [ searcher acceptToken: token ] ] ].
	self items addAll: (searcher select: [ :structure | 
		structure parents allSatisfy: [ :parent | 
			(self context structure: parent) isValid ] ])! !

!PRSearchView methodsFor: 'rendering' stamp: 'lr 9/12/2009 18:54'!
renderContentOn: html
	self hasSearchForm
		ifTrue: [ self renderFormOn: html ].
	(self query isNil or: [ self items notEmpty ])
		ifTrue: [ self renderResultOn: html ]
		ifFalse: [ self renderNoResultOn: html ]! !

!PRSearchView methodsFor: 'rendering' stamp: 'lr 9/12/2009 16:54'!
renderFormOn: html
	html form class: 'search-form'; with: [
		html label: [
			html span: 'Enter your keywords: '.
			html textInput
				value: self query;
				callback: [ :value | self query: value ].
			html submitButton: 'Search' ] ]! !

!PRSearchView methodsFor: 'rendering' stamp: 'lr 9/12/2009 16:50'!
renderNoResultOn: html
	html paragraph: 'Your search yielded no results.'.
	html unorderedList: [
		html listItem: 'Check if your spelling is correct.'.
		html listItem: 'Consider loosening your query by removing words.' ]! !

!PRSearchView methodsFor: 'rendering' stamp: 'lr 9/12/2009 16:53'!
renderResultOn: html
	self items isEmpty 
		ifTrue: [ ^ self ].
	html definitionList class: 'search-result'; with: [
		self items do: [ :structure |
			html definitionTerm: [
				html anchor
					goto: (self context structure: structure);
					with: structure title ].
			html definitionData: (self summaryOf: structure) ] ]! !

!PRSearchView methodsFor: 'private' stamp: 'lr 9/12/2009 12:43'!
summaryOf: aStructure
	| input output |
	aStructure description do: [ :description |
		description isDocumentDescription ifTrue: [
			output := WriteStream on: String new.
			input := (PRTextWriter write: (aStructure readUsing: description)) readStream.
			[ input atEnd ] whileFalse: [
				output nextPutAll: (input upTo: $<).
				input upTo: $> ].
			self summarySize isNil ifFalse: [
				output position: (output size min: self summarySize).
				output atEnd
					ifFalse: [ output nextPutAll: '...' ] ].
			^ output contents ] ].
	^ nil! !

!PRSearchView methodsFor: 'configuration' stamp: 'lr 9/12/2009 12:42'!
summarySize
	^ 256! !

!PRSearchView methodsFor: 'updating' stamp: 'lr 2/23/2010 21:14'!
updateUrl: anUrl 
	super updateUrl: anUrl.
	self hasValidQuery 
		ifTrue: [ anUrl queryFields at: self queryFieldName put: self query ]! !

!PRViewComponent class methodsFor: 'testing' stamp: 'lr 3/17/2009 17:28'!
isListed
	^ true! !

!PRViewComponent class methodsFor: 'testing' stamp: 'lr 6/11/2009 09:22'!
isValidIn: aContext
	"Verify that a view as a meaning for the context aContext. Override this message, to add new tests, but do not forget to call this super implementation."

	^ self isAbstract not and: [ aContext structure isValidView: self in: aContext ]! !

!PRViewComponent class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:30'!
on: aContext
	^ self new
		setContext: aContext;
		yourself! !

!PRViewComponent methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
context
	^ context! !

!PRViewComponent methodsFor: 'accessing' stamp: 'lr 9/12/2007 18:18'!
cssClass
	"Answer a CSS class that should be used in the body-tag when this view is displayed."

	^ self class fullName asLowercase allButFirst: 2! !

!PRViewComponent methodsFor: 'private' stamp: 'lr 5/2/2009 10:05'!
find: aStructureClass
	^ self context structure findAncestor: [ :each | each isKindOf: aStructureClass ]! !

!PRViewComponent methodsFor: 'responding' stamp: 'jf 11/4/2009 11:46'!
handle: aRequestContext
	self isFullResponse ifFalse: [ self shouldNotImplement ].
	aRequestContext respond: [ :response | self respondUsing: response ]! !

!PRViewComponent methodsFor: 'testing' stamp: 'jf 11/4/2009 11:04'!
isFullResponse
	"Returns true if the View generates a complete HTTP response."
	
	^ false! !

!PRViewComponent methodsFor: 'testing' stamp: 'lr 7/13/2009 20:55'!
isRestful
	^ false! !

!PRViewComponent methodsFor: 'accessing' stamp: 'lr 1/29/2008 09:06'!
label
	^ self class label! !

!PRViewComponent methodsFor: 'rendering' stamp: 'lr 3/9/2006 11:30'!
renderContentOn: html
	html strong: self context! !

!PRViewComponent methodsFor: 'accessing' stamp: 'lr 4/8/2009 20:17'!
rendererInstance
	^ PRViewRenderer new! !

!PRViewComponent methodsFor: 'responding' stamp: 'jf 11/4/2009 11:54'!
respondUsing: aResponse
	"Allows views that return a full response to configure the Response."! !

!PRViewComponent methodsFor: 'accessing' stamp: 'lr 2/28/2007 16:04'!
robots
	^ 'noindex,nofollow'! !

!PRViewComponent methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:30'!
setContext: aContext
	context := aContext! !

!PRViewComponent methodsFor: 'updating' stamp: 'lr 8/11/2007 08:28'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot bodyAttributes addClass: self cssClass! !

MAComponent subclass: #PRWidget
	instanceVariableNames: 'properties context owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

PRWidget subclass: #PRBatcherWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRBatcherWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 3/26/2009 22:34'!
descriptionShowFirstLast
	^ MABooleanDescription new
		parameterName: 'show-first-last';
		checkboxLabel: 'First/Last';
		accessor: #showFirstLast;
		label: 'Show Links';
		default: true;
		priority: 100;
		yourself! !

!PRBatcherWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 3/26/2009 22:34'!
descriptionShowNumbers
	^ MABooleanDescription new
		parameterName: 'show-numbers';
		checkboxLabel: 'Numbers';
		accessor: #showNumbers;
		default: true;
		priority: 230;
		yourself! !

!PRBatcherWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 3/26/2009 22:34'!
descriptionShowPreviousNext
	^ MABooleanDescription new
		parameterName: 'show-previous-next';
		checkboxLabel: 'Previous/Next';
		accessor: #showPreviousNext;
		default: true;
		priority: 210;
		yourself! !

!PRBatcherWidget class methodsFor: 'testing' stamp: 'lr 8/22/2007 15:25'!
isAbstract
	^ false! !

!PRBatcherWidget class methodsFor: 'accessing' stamp: 'lr 8/22/2007 15:32'!
label
	^ 'Batcher'! !

!PRBatcherWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 18:18'!
nextSibling
	"Answer the next sibling in the current context or nil."

	^ self siblings after: self context structure ifAbsent: [ nil ]! !

!PRBatcherWidget methodsFor: 'accessing' stamp: 'lr 8/22/2007 15:26'!
previousSibling
	"Answer the previous sibling in the current context or nil."

	^ self siblings before: self context structure ifAbsent: [ nil ]! !

!PRBatcherWidget methodsFor: 'rendering' stamp: 'lr 8/24/2008 11:56'!
renderContentOn: html 
	self siblings isEmpty ifTrue: [ ^ self ].
	(self read: #descriptionShowFirstLast) ifTrue: 
		[ self 
			renderLabel: '|<'
			class: 'first'
			structure: self siblings first
			on: html ].
	(self read: #descriptionShowPreviousNext) ifTrue: 
		[ self 
			renderLabel: '<<'
			class: 'previous'
			structure: self previousSibling
			on: html ].
	(self read: #descriptionShowNumbers) ifTrue: 
		[ self siblings keysAndValuesDo: 
			[ :index :each | 
			self 
				renderLabel: index
				class: 'number'
				structure: each
				on: html ] ].
	(self read: #descriptionShowPreviousNext) ifTrue: 
		[ self 
			renderLabel: '>>'
			class: 'next'
			structure: self nextSibling
			on: html ].
	(self read: #descriptionShowFirstLast) ifTrue: 
		[ self 
			renderLabel: '>|'
			class: 'last'
			structure: self siblings last
			on: html ]! !

!PRBatcherWidget methodsFor: 'rendering' stamp: 'lr 9/12/2007 18:18'!
renderLabel: aString class: aCssString structure: aStructure on: html
	| active |
	active := self context structure = aStructure.
	(active or: [ aStructure isNil ])
		ifTrue: [
			html span
				class: aCssString;
				class: 'active' if: active;
				with: aString ]
		ifFalse: [
			html anchor
				class: aCssString;
				title: aStructure title;
				goto: (self context structure: aStructure);
				with: aString ].
	html text: ' '! !

!PRBatcherWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 18:18'!
siblings
	"Answer an list of siblings in the current context."

	^ (self context enumeratorFor: self context structure parent) contents! !

PRWidget subclass: #PRContentsWidget
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRContentsWidget class methodsFor: 'descriptions' stamp: 'lr 4/10/2008 14:17'!
descriptionHeadAuthor
	^ MAStringDescription new
		comment: 'The author to appear in the html head.';
		accessor: #headAuthor;
		label: 'Head Author';
		priority: 220;
		beSetting;
		yourself! !

!PRContentsWidget class methodsFor: 'descriptions' stamp: 'lr 4/10/2008 14:17'!
descriptionHeadKeywords
	^ MAStringDescription new
		comment: 'The keywords to appear in the html head.';
		accessor: #headKeywords;
		label: 'Head Keywords';
		default: '%k %t';
		priority: 210;
		beSetting;
		yourself! !

!PRContentsWidget class methodsFor: 'descriptions' stamp: 'lr 4/10/2008 14:17'!
descriptionHeadTitle
	^ MAStringDescription new
		comment: 'The title to appear in the browser window.';
		accessor: #headTitle;
		label: 'Head Title';
		default: '%k: %t';
		priority: 200;
		beSetting;
		yourself! !

!PRContentsWidget class methodsFor: 'descriptions' stamp: 'lr 4/10/2008 14:17'!
descriptionHeading
	^ MAStringDescription new
		comment: 'The header to appear on before the page contents.';
		accessor: #heading;
		label: 'Heading';
		priority: 110;
		yourself! !

!PRContentsWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRContentsWidget class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Contents'! !

!PRContentsWidget methodsFor: 'private' stamp: 'NickAger 2/26/2010 19:06'!
buildComponent: aContext
	^ aContext command asComponent
! !

!PRContentsWidget methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:30'!
children
	^ Array with: self component! !

!PRContentsWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
component
	^ component ifNil: [ component := self buildComponent: self context ]! !

!PRContentsWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
component: aComponent
	component := aComponent! !

!PRContentsWidget methodsFor: 'accessing-properties' stamp: 'lr 10/24/2007 20:36'!
headAuthor
	^ self expand: (self read: #descriptionHeadAuthor)! !

!PRContentsWidget methodsFor: 'accessing-properties' stamp: 'lr 10/24/2007 20:38'!
headKeywords
	^ self expand: (self read: #descriptionHeadKeywords)! !

!PRContentsWidget methodsFor: 'accessing-properties' stamp: 'lr 10/24/2007 20:36'!
headTitle
	^ self expand: (self read: #descriptionHeadTitle)! !

!PRContentsWidget methodsFor: 'accessing-properties' stamp: 'lr 4/23/2007 08:04'!
heading
	^ self expand: (self read: #descriptionHeading)! !

!PRContentsWidget methodsFor: 'events' stamp: 'lr 4/20/2007 14:23'!
onAnswerCommand: aCommand
	aCommand isNil
		ifTrue: [ ^ self context: (self context structure: self context structure) ].
	[ aCommand execute ]
		on: Error
		do: [ :err | ^ self component errors add: err ].
	self context: aCommand answer! !

!PRContentsWidget methodsFor: 'events' stamp: 'NickAger 2/26/2010 19:12'!
onChangeContext: aContext
	super onChangeContext: aContext.
	aContext command isQuick
		ifTrue: [ self onAnswerCommand: aContext command ]
		ifFalse: [
			self component: (self buildComponent: aContext).
			aContext command isView ifFalse: [
				self component onAnswer: [ :value | self onAnswerCommand: value ]
				]
		]! !

!PRContentsWidget methodsFor: 'rendering' stamp: 'lr 10/28/2008 12:21'!
renderContentOn: html
	PRContentsRendered value
		ifTrue: [ ^ self ].
	PRContentsRendered use: true during: [
		self heading isEmptyOrNil ifFalse: [
			html heading
				level: 1;
				class: 'heading';
				with: self heading ].
		html render: self component ]! !

!PRContentsWidget methodsFor: 'accessing-dynamic' stamp: 'lr 5/12/2007 11:40'!
states
	^ Array with: self! !

!PRContentsWidget methodsFor: 'updating' stamp: 'lr 11/3/2007 09:51'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot title: self headTitle.
	self context command updateRoot: aHtmlRoot.
	self headAuthor isEmptyOrNil ifFalse: [
		aHtmlRoot meta
			name: 'author';
			content: self headAuthor ].
	self headKeywords isEmptyOrNil ifFalse: [
		aHtmlRoot meta
			name: 'keywords';
			content: self headKeywords ]! !

PRWidget subclass: #PRFileSettings
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets-Settings'!

!PRFileSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRFileSettings class methodsFor: 'testing' stamp: 'lr 2/13/2009 20:33'!
isAbstract
	^ false! !

!PRFileSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:33'!
label
	^ 'File Settings'! !

!PRFileSettings methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:40'!
children
	^ Array with: component! !

!PRFileSettings methodsFor: 'accessing' stamp: 'jf 9/28/2009 13:34'!
description
	^ MAContainer new
		add: (MAStringDescription new
			label: 'Base Directory';
			accessor: #baseDirectoryPath;
			comment: 'Defines the base-directory where the files are stored. If this field is left empty, it default to a subdirectory of of the current image-location.';
			yourself);
		add: (MAStringDescription new
			label: 'Base URL';
			accessor: #baseUrl;
			comment: 'Defines the base-URL where the files are served from, when using an external web server. if this field is left empty, files are served through the image.';
			yourself);
		yourself! !

!PRFileSettings methodsFor: 'initialization' stamp: 'lr 2/13/2009 20:57'!
initialize
	super initialize.
	component := (self description asComponentOn: self)
		addValidatedForm;
		yourself! !

!PRFileSettings methodsFor: 'accessing' stamp: 'jf 9/28/2009 11:17'!
model
	^ PRFile fileModel! !

!PRFileSettings methodsFor: 'dispatching' stamp: 'jf 9/28/2009 13:34'!
readUsing: aDescription
	^ self model readUsing: aDescription
		! !

!PRFileSettings methodsFor: 'rendering' stamp: 'lr 2/13/2009 20:40'!
renderContentOn: html
	html render: self children! !

!PRFileSettings methodsFor: 'dispatching' stamp: 'lr 2/13/2009 21:02'!
write: anObject using: aDescription
	^ self model write: anObject using: aDescription! !

PRWidget subclass: #PRHtmlWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRHtmlWidget class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultText
	^ '<h1>%k</h1><h2>%c</h2>'! !

!PRHtmlWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 4/4/2009 18:26'!
descriptionText
	^ MAMemoDescription new
		default: self defaultText;
		parameterName: 'text';
		accessor: #text;
		priority: 100;
		label: 'Text';
		yourself! !

!PRHtmlWidget class methodsFor: 'testing' stamp: 'lr 9/3/2008 13:55'!
isAbstract
	^ false! !

!PRHtmlWidget class methodsFor: 'accessing' stamp: 'lr 7/5/2007 22:37'!
label
	^ 'HTML'! !

!PRHtmlWidget methodsFor: 'rendering' stamp: 'lr 7/5/2007 22:41'!
renderContentOn: html
	html html: self text! !

!PRHtmlWidget methodsFor: 'accessing' stamp: 'lr 7/5/2007 22:37'!
text
	^ self expand: (self read: #descriptionText)! !

PRWidget subclass: #PRKernelSettings
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets-Settings'!

!PRKernelSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRKernelSettings class methodsFor: 'testing' stamp: 'lr 9/3/2008 13:55'!
isAbstract
	^ false! !

!PRKernelSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:24'!
label
	^ 'Kernel Settings'! !

!PRKernelSettings methodsFor: 'accessing' stamp: 'lr 9/3/2008 13:53'!
children
	^ Array with: component! !

!PRKernelSettings methodsFor: 'initialization' stamp: 'lr 9/3/2008 13:51'!
initialize
	super initialize.
	component := self kernel asComponent
		addValidatedForm;
		yourself! !

!PRKernelSettings methodsFor: 'rendering' stamp: 'lr 9/3/2008 13:56'!
renderContentOn: html
	html render: self children! !

PRWidget subclass: #PRListWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

PRListWidget subclass: #PRCommandsWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRCommandsWidget class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultCommandClasses
	^ (Array withAll: PRCommand withAllConcreteClasses)
		sort: [ :a :b | a label caseInsensitiveLessOrEqual: b label ];
		yourself! !

!PRCommandsWidget class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionCommandClasses
	^ MAMultipleOptionDescription new
		default: [ self defaultCommandClasses ] magritteDynamicObject;
		options: [ self defaultCommandClasses ] magritteDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: MAClassDescription new;
		accessor: #commandClasses;
		label: 'Command Classes';
		priority: 200;
		beOrdered;
		beDistinct;
		beSorted;
		yourself! !

!PRCommandsWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRCommandsWidget class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Commands'! !

!PRCommandsWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:35'!
commandClasses
	^ self read: #descriptionCommandClasses! !

!PRCommandsWidget methodsFor: 'accessing' stamp: 'lr 8/15/2007 20:51'!
items
	| commands |
	commands := self context commands.
	^ self commandClasses select: [ :each | commands includes: each ]! !

!PRCommandsWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:58'!
labelFor: anObject
	^ anObject shortLabel! !

!PRCommandsWidget methodsFor: 'rendering' stamp: 'lr 4/18/2007 14:58'!
renderItem: aClass on: html
	html anchor
		class: 'active' if: (self isActive: aClass);
		accessKey: aClass accessKey;
		goto: (self context
			structure: self context structure
			command: aClass new);
		with: (self labelFor: aClass)! !

!PRCommandsWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
selected
	^ self context command class! !

!PRListWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier List'! !

!PRListWidget methodsFor: 'testing' stamp: 'lr 2/9/2007 15:00'!
isActive: anObject
	^ self selected = anObject! !

!PRListWidget methodsFor: 'accessing' stamp: 'lr 5/28/2007 16:31'!
items
	"Answer the list items."

	self subclassResponsibility! !

!PRListWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:57'!
labelFor: anObject
	^ anObject! !

!PRListWidget methodsFor: 'rendering' stamp: 'lr 2/21/2010 13:17'!
renderContentOn: html
	| items |
	items := self items.
	items isEmpty
		ifFalse: [ self renderItems: items on: html ]! !

!PRListWidget methodsFor: 'rendering' stamp: 'lr 4/18/2007 14:57'!
renderItem: anObject on: html
	html render: (self labelFor: anObject)! !

!PRListWidget methodsFor: 'rendering' stamp: 'lr 12/18/2007 21:45'!
renderItems: aCollection on: html
	html unorderedList: [
		aCollection do: [ :each |
			html listItem: [
				self renderItem: each on: html ] ] ]! !

!PRListWidget methodsFor: 'accessing' stamp: 'lr 5/28/2007 16:31'!
selected
	"Answer the selected list item."

	self subclassResponsibility! !

PRListWidget subclass: #PRStructuresWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

PRStructuresWidget subclass: #PRAjaxSearchWidget
	instanceVariableNames: 'searchText items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRAjaxSearchWidget class methodsFor: 'accessing-defaults' stamp: 'lr 9/12/2009 12:49'!
defaultMaxItems
	^ 10! !

!PRAjaxSearchWidget class methodsFor: 'accessing-description' stamp: 'lr 9/12/2009 12:49'!
descriptionDropdownList
	^ nil! !

!PRAjaxSearchWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 4/10/2008 14:17'!
descriptionExampleText
	^ MAStringDescription new
		accessor: #exampleText;
		default: 'Search';
		label: 'Example';
		priority: 400;
		yourself! !

!PRAjaxSearchWidget class methodsFor: 'accessing' stamp: 'lr 9/12/2009 12:49'!
groupLabel
	^ 'Pier Search'! !

!PRAjaxSearchWidget class methodsFor: 'testing' stamp: 'lr 9/12/2009 12:49'!
isAbstract
	^ false! !

!PRAjaxSearchWidget class methodsFor: 'accessing' stamp: 'lr 9/12/2009 12:55'!
label
	^ 'Search Field (AJAX)'! !

!PRAjaxSearchWidget methodsFor: 'accessing-dynamic' stamp: 'lr 9/12/2009 12:49'!
dropdownList
	^ false! !

!PRAjaxSearchWidget methodsFor: 'accessing' stamp: 'lr 6/24/2007 09:51'!
exampleText
	^ self read: #descriptionExampleText! !

!PRAjaxSearchWidget methodsFor: 'private' stamp: 'lr 9/12/2009 12:50'!
findItems
	^ (self context root fullTextSearch subString: self searchText) contents! !

!PRAjaxSearchWidget methodsFor: 'rendering' stamp: 'NickAger 1/25/2010 16:45'!
renderContentOn: html
	| inputId autoId |
	html textInput
		id: (inputId := html nextId);
		exampleText: self exampleText;
		onEnter: (html scriptaculous autocompleter
			alias: (autoId := html nextId);
			activate);
		callback: [ :value | self searchText: value ].
	html div 
		script: (html scriptaculous autocompleter
			assignTo: autoId;
			element: inputId;
			onSelected: (self selectionHandlerOn: html);
			on: #renderItemsOn:for: of: self)! !

!PRAjaxSearchWidget methodsFor: 'rendering' stamp: 'NickAger 1/25/2010 16:46'!
renderItemsOn: html for: aString
	self searchText: aString.
	 
	html unorderedList: [
		self items do: [ :each |
			html listItem
				passenger: each; " the only difference to the original "
				with: [ self renderItem: each on: html ] ] ]! !

!PRAjaxSearchWidget methodsFor: 'accessing' stamp: 'lr 9/12/2009 12:49'!
searchText
	^ searchText ifNil: [ searchText := String new ]! !

!PRAjaxSearchWidget methodsFor: 'accessing' stamp: 'lr 9/12/2009 12:49'!
searchText: aString
	searchText := aString.
	items := nil! !

!PRAjaxSearchWidget methodsFor: 'private' stamp: 'jf 9/24/2009 15:13'!
selectionHandlerOn: html
	| structure |
	structure := WAValueHolder new.
	^ html scriptaculous evaluator
		callback: [ :value | structure contents: (html context callbacks passengerAt: value) ]
			value: (JSStream on: 'arguments[1].id');
		callback: [ :script |
			self searchText: String new.
			structure contents isNil ifFalse: [
				script goto: ((self context structure: structure contents)
					urlOn: html) ] ]! !

!PRAjaxSearchWidget methodsFor: 'accessing-dynamic' stamp: 'lr 9/12/2009 12:49'!
unfilteredItems
	^ items ifNil: [ items := self findItems ]! !

PRStructuresWidget subclass: #PRBreadcrumbWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRBreadcrumbWidget class methodsFor: 'accessing-defaults' stamp: 'lr 6/27/2007 12:23'!
defaultAllButLast
	^ false! !

!PRBreadcrumbWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 3/26/2009 22:35'!
descriptionAllButLast
	^ MABooleanDescription new
		default: self defaultAllButLast;
		parameterName: 'all-but-last';
		accessor: #allButLast;
		label: 'All But Last';
		priority: 300;
		yourself! !

!PRBreadcrumbWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRBreadcrumbWidget class methodsFor: 'accessing' stamp: 'lr 8/29/2008 11:51'!
label
	^ 'Breadcrumb'! !

!PRBreadcrumbWidget methodsFor: 'accessing' stamp: 'lr 6/27/2007 12:25'!
allButLast
	^ self read: #descriptionAllButLast! !

!PRBreadcrumbWidget methodsFor: 'accessing' stamp: 'lr 6/27/2007 12:26'!
unfilteredItems
	| items |
	items := self context structure parents.
	self allButLast ifTrue: [ items := items allButLast ].
	^ items! !

PRStructuresWidget subclass: #PRChildrenWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRChildrenWidget class methodsFor: 'accessing-defaults' stamp: 'lr 7/11/2007 11:57'!
defaultExpand
	^ false! !

!PRChildrenWidget class methodsFor: 'accessing-defaults' stamp: 'lr 2/3/2007 13:01'!
defaultLevel
	^ nil! !

!PRChildrenWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:31'!
descriptionExpand
	^ MABooleanDescription new
		default: self defaultExpand;
		parameterName: 'expand';
		accessor: #expand;
		label: 'Expand';
		priority: 310;
		yourself! !

!PRChildrenWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:31'!
descriptionLevel
	^ MANumberDescription new
		default: self defaultLevel;
		parameterName: 'level';
		accessor: #level;
		label: 'Level';
		priority: 300;
		beInteger;
		min: 0;
		yourself! !

!PRChildrenWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRChildrenWidget class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Children'! !

!PRChildrenWidget methodsFor: 'accessing' stamp: 'lr 7/11/2007 12:03'!
expand
	^ self read: #descriptionExpand! !

!PRChildrenWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:15'!
level
	^ self read: #descriptionLevel! !

!PRChildrenWidget methodsFor: 'rendering' stamp: 'lr 9/25/2007 08:02'!
renderItem: aStrucure on: html
	| children |
	super renderItem: aStrucure on: html.
	self expand ifFalse: [ ^ self ].
	children := self childrenOf: aStrucure.
	children isEmpty
		ifFalse: [ self renderItems: children on: html ]! !

!PRChildrenWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 18:18'!
unfilteredItems
	| structure children |
	structure :=  self context structure.
	self level isNil ifFalse: [
		^ self childrenOf: (structure parents
			at: self level
			ifAbsent: [ ^ #() ]) ].
	children := self childrenOf: structure.
	^ (children isEmpty and: [ structure isRoot not ])
		ifTrue: [ self childrenOf: structure parent ]
		ifFalse: [ children ]! !

PRStructuresWidget subclass: #PRNavigationWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRNavigationWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRNavigationWidget class methodsFor: 'accessing' stamp: 'lr 2/14/2008 14:19'!
label
	^ 'Navigation'! !

!PRNavigationWidget methodsFor: 'rendering' stamp: 'lr 4/23/2007 07:44'!
renderContentOn: html
	self renderParents: self context structure parents on: html! !

!PRNavigationWidget methodsFor: 'rendering' stamp: 'lr 2/9/2007 19:37'!
renderParents: aCollection on: html
	html unorderedList: [
		html listItem: [
			self renderItem: aCollection first on: html.
			(aCollection size = 1 or: [ aCollection size = 2 and: [ (self childrenOf: aCollection last) isEmpty ] ])
				ifTrue: [ self renderItems: (self childrenOf: aCollection first) on: html ]
				ifFalse: [ self renderParents: aCollection allButFirst on: html ] ] ]! !

PRStructuresWidget subclass: #PRReferencesWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRReferencesWidget class methodsFor: 'accessing-defaults' stamp: 'lr 2/9/2007 14:48'!
defaultMaxItems
	^ 10! !

!PRReferencesWidget class methodsFor: 'testing' stamp: 'lr 4/16/2007 13:39'!
isAbstract
	^ false! !

!PRReferencesWidget class methodsFor: 'accessing' stamp: 'lr 4/16/2007 13:39'!
label
	^ 'References'! !

!PRReferencesWidget methodsFor: 'accessing' stamp: 'lr 4/29/2009 16:42'!
unfilteredItems
	^ ((self context root incomingReferences
		to: self context structure;
		collect: [ :each | each owner ])
		select: [ :each | each notNil ])
			asSet! !

!PRStructuresWidget class methodsFor: 'accessing-defaults' stamp: 'lr 4/18/2007 14:01'!
defaultDropdownList
	^ false! !

!PRStructuresWidget class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultFilter
	^ Array new! !

!PRStructuresWidget class methodsFor: 'accessing-defaults' stamp: 'lr 5/3/2006 17:28'!
defaultLabel
	^ self defaultLabels first! !

!PRStructuresWidget class methodsFor: 'accessing-defaults' stamp: 'lr 6/22/2007 18:34'!
defaultLabels
	^ #( menu title name )! !

!PRStructuresWidget class methodsFor: 'accessing-defaults' stamp: 'lr 2/3/2007 13:07'!
defaultMaxItems
	^ nil! !

!PRStructuresWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:35'!
descriptionDropdownList
	^ MABooleanDescription new
		comment: 'Displays the structures as a dropdown list';
		default: self defaultDropdownList;
		parameterName: 'dropdown-list';
		accessor: #dropdownList;
		label: 'Dropdown List';
		priority: 270;
		yourself! !

!PRStructuresWidget class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionFilter
	^ MAMultipleOptionDescription new
		default: [ PRStructure withAllConcreteClasses ] magritteDynamicObject;
		options: [ PRStructure withAllConcreteClasses ] magritteDynamicObject;
		comment: 'The types of structures not to display.';
		componentClass: MAMultiselectListComponent;
		reference: MAClassDescription new;
		default: self defaultFilter;
		accessor: #filter;
		label: 'Filter';
		priority: 210;
		beSorted;
		yourself! !

!PRStructuresWidget class methodsFor: 'accessing-description' stamp: 'lr 11/25/2008 09:22'!
descriptionLabel
	^ MASingleOptionDescription new
		comment: 'The label to be used to name structures.';
		options: self defaultLabels;
		default: self defaultLabel;
		accessor: #label;
		label: 'Label';
		priority: 200;
		beRequired;
		beSorted;
		yourself! !

!PRStructuresWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:35'!
descriptionMaxItems
	^ MANumberDescription new
		comment: 'The maximum number of items to be displayed.';
		default: self defaultMaxItems;
		parameterName: 'max-items';
		accessor: #maxItems;
		label: 'Max Items';
		priority: 230;
		beInteger;
		min: 1;
		yourself! !

!PRStructuresWidget methodsFor: 'private' stamp: 'lr 4/23/2007 07:53'!
childrenOf: aStructure
	^ (self context enumeratorFor: aStructure) reject: [ :each | self isFiltered: each ]! !

!PRStructuresWidget methodsFor: 'accessing-settings' stamp: 'lr 4/18/2007 14:45'!
dropdownList
	^ self read: #descriptionDropdownList! !

!PRStructuresWidget methodsFor: 'accessing-settings' stamp: 'lr 4/18/2007 14:13'!
filter
	^ self read: #descriptionFilter! !

!PRStructuresWidget methodsFor: 'testing' stamp: 'lr 11/25/2008 09:17'!
isFiltered: aStructure
	"Test if aStructure should be filtered from the widget:
	
	(1) Do not display if it is hidden from the menus.
	(2) Do not display if itself or one of the parents has an invalid view.
	(3) Do not display if it is one of the filtered classses."

	aStructure hideFromMenus
		ifTrue: [ ^ true ].
	aStructure parents do: [ :parent |
		(self context structure: parent) isValid
			ifFalse: [ ^ true ] ].
	^ self filter 
		anySatisfy: [ :each | aStructure class = each ]! !

!PRStructuresWidget methodsFor: 'testing' stamp: 'lr 2/14/2009 16:55'!
isPath: aStructure
	"Test if aStructure is within the path of the currently active structure."
	
	^ (self isActive: aStructure) not and: [ self selected parents includes: aStructure ]! !

!PRStructuresWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:49'!
items
	| items |
	items := self unfilteredItems asArray
		reject: [ :each | self isFiltered: each ].
	self maxItems isNil
		ifFalse: [ items := items first: (items size min: self maxItems) ].
	^ items! !

!PRStructuresWidget methodsFor: 'accessing-settings' stamp: 'lr 4/18/2007 14:14'!
label
	^ self read: #descriptionLabel! !

!PRStructuresWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:58'!
labelFor: aStructure
	^ aStructure perform: self label! !

!PRStructuresWidget methodsFor: 'accessing-settings' stamp: 'lr 4/18/2007 14:14'!
maxItems
	^ self read: #descriptionMaxItems! !

!PRStructuresWidget methodsFor: 'rendering' stamp: 'lr 5/28/2007 16:31'!
renderContentOn: html
	self dropdownList
		ifTrue: [ self renderSelect: self items on: html ]
		ifFalse: [ super renderContentOn: html ]! !

!PRStructuresWidget methodsFor: 'rendering' stamp: 'lr 5/24/2009 19:38'!
renderItem: aStructure on: html
	html anchor
		class: 'path' if: (self isPath: aStructure);
		class: 'active' if: (self isActive: aStructure);
		goto: (self context structure: aStructure);
		with: [ html html: (self labelFor: aStructure) ]! !

!PRStructuresWidget methodsFor: 'rendering' stamp: 'lr 4/18/2007 15:45'!
renderSelect: aCollection on: html
	html form: [
		html select
			list: aCollection;
			beSubmitOnChange;
			selected: self selected;
			labels: [ :each | self labelFor: each ];
			callback: [ :each | self context: (self context structure: each) ] ]! !

!PRStructuresWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
selected
	^ self context structure! !

!PRStructuresWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
unfilteredItems
	^ self subclassResponsibility! !

PRStructuresWidget subclass: #PRTreeWidget
	instanceVariableNames: 'expanded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRTreeWidget class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultAutoCollapse
	^ true! !

!PRTreeWidget class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultAutoExpand
	^ true! !

!PRTreeWidget class methodsFor: 'accessing-defaults' stamp: 'lr 9/16/2009 11:58'!
defaultDisplayRoot
	^ true! !

!PRTreeWidget class methodsFor: 'accessing-defaults' stamp: 'lr 9/16/2009 11:46'!
defaultLevel
	^ nil! !

!PRTreeWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:32'!
descriptionAutoCollapse
	^ MABooleanDescription new
		comment: 'Automatically collaps structures not currently active.';
		default: self defaultAutoCollapse;
		parameterName: 'auto-collapse';
		accessor: #autoCollapse;
		label: 'Auto Collapse';
		priority: 310;
		yourself! !

!PRTreeWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:32'!
descriptionAutoExpand
	^ MABooleanDescription new
		comment: 'Automatically expand to current structure.';
		default: self defaultAutoExpand;
		parameterName: 'auto-expand';
		accessor: #autoExpand;
		label: 'Auto Expand';
		priority: 300;
		yourself! !

!PRTreeWidget class methodsFor: 'accessing-description' stamp: 'lr 9/16/2009 11:58'!
descriptionDisplayRoot
	^ MABooleanDescription new
		comment: 'Whether the root node of the tree should be displayed';
		default: self defaultDisplayRoot;
		parameterName: 'display-root';
		accessor: #displayRoot;
		label: 'Display Root';
		priority: 320;
		yourself! !

!PRTreeWidget class methodsFor: 'accessing-description' stamp: 'lr 4/18/2007 15:05'!
descriptionDropdownList
	^ nil! !

!PRTreeWidget class methodsFor: 'accessing-description' stamp: 'lr 9/16/2009 11:57'!
descriptionLevel
	^ MANumberDescription new
		comment: 'The level from which the tree should be displayed.';
		default: self defaultLevel;
		parameterName: 'level';
		accessor: #level;
		label: 'Level';
		priority: 230;
		beInteger;
		min: 0;
		yourself! !

!PRTreeWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRTreeWidget class methodsFor: 'converting' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Tree'! !

!PRTreeWidget methodsFor: 'accessing-properties' stamp: 'lr 4/18/2007 14:16'!
autoCollapse
	^ self read: #descriptionAutoCollapse! !

!PRTreeWidget methodsFor: 'accessing-properties' stamp: 'lr 4/18/2007 14:16'!
autoExpand
	^ self read: #descriptionAutoExpand! !

!PRTreeWidget methodsFor: 'actions' stamp: 'lr 3/9/2006 11:30'!
collapseStructure: aStructure
	expanded remove: aStructure ifAbsent: [ "should never happen" ]! !

!PRTreeWidget methodsFor: 'accessing-properties' stamp: 'lr 9/16/2009 11:58'!
displayRoot
	^ self read: #descriptionDisplayRoot! !

!PRTreeWidget methodsFor: 'accessing-properties' stamp: 'lr 4/18/2007 15:14'!
dropdownList
	^ false! !

!PRTreeWidget methodsFor: 'actions' stamp: 'lr 3/9/2006 11:30'!
expandStructure: aStructure
	expanded add: aStructure! !

!PRTreeWidget methodsFor: 'initialization' stamp: 'lr 5/12/2007 11:40'!
initialize
	super initialize.
	expanded := Set new! !

!PRTreeWidget methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isExpanded: aNode
	^ expanded includes: aNode! !

!PRTreeWidget methodsFor: 'accessing-properties' stamp: 'lr 9/16/2009 11:46'!
level
	^ self read: #descriptionLevel! !

!PRTreeWidget methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
onChangeContext: aContext
	super onChangeContext: aContext.
	self autoCollapse
		ifTrue: [ expanded removeAll: expanded copy ].
	self autoExpand
		ifTrue: [ expanded addAll: aContext structure parents ]! !

!PRTreeWidget methodsFor: 'rendering' stamp: 'lr 9/16/2009 11:59'!
renderContentOn: html
	| root |
	root := self context root.
	self level notNil ifTrue: [
		root := self context structure parents
			at: self level
			ifAbsent: [ ^ self ] ].
	self displayRoot
		ifFalse: [ self renderStructureChildren: root on: html ]
		ifTrue: [ html unorderedList: [ self renderStructure: root on: html ] ]! !

!PRTreeWidget methodsFor: 'rendering-items' stamp: 'lr 2/12/2007 19:07'!
renderStructure: aStructure on: html
	html listItem: [
		self renderStructureButton: aStructure on: html.
		self renderStructureLabel: aStructure on: html.
		self renderStructureChildren: aStructure on: html ]! !

!PRTreeWidget methodsFor: 'rendering-items' stamp: 'lr 5/28/2007 16:28'!
renderStructureButton: aStructure on: html
	| isExpanded |
	html span class: 'button'; with: [
		(self childrenOf: aStructure) isEmpty
			ifTrue: [ html space ]
			ifFalse: [
				isExpanded := self isExpanded: aStructure.
				html anchor
					callback: [
						isExpanded
							ifTrue: [ self collapseStructure: aStructure ]
							ifFalse: [ self expandStructure: aStructure ] ];
					with: (isExpanded
						ifTrue: [ '-' ]
						ifFalse: [ '+' ]) ] ]! !

!PRTreeWidget methodsFor: 'rendering-items' stamp: 'lr 3/9/2006 11:30'!
renderStructureChildren: aStructure on: html
	| children |
	(self isExpanded: aStructure)
		ifFalse: [ ^ self ].
	children := self childrenOf: aStructure.
	children isEmpty ifTrue: [ ^ self ].
	html unorderedList: [
		children do: [ :each |
			self renderStructure: each on: html ] ]! !

!PRTreeWidget methodsFor: 'rendering-items' stamp: 'lr 3/9/2006 11:30'!
renderStructureLabel: aStructure on: html
	self renderItem: aStructure on: html! !

!PRTreeWidget methodsFor: 'accessing' stamp: 'lr 5/12/2007 11:40'!
states
	^ Array with: expanded! !

!PRTreeWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
unfilteredItems
	self shouldNotImplement! !

PRStructuresWidget subclass: #PRVisitedWidget
	instanceVariableNames: 'structures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRVisitedWidget class methodsFor: 'defaults' stamp: 'lr 8/29/2008 11:13'!
defaultMaxItems
	^ 10! !

!PRVisitedWidget class methodsFor: 'accessing-description' stamp: 'lr 3/26/2009 22:33'!
descriptionRemoveDuplicates
	^ MABooleanDescription new
		comment: 'Avoid duplicates in the list';
		parameterName: 'remove-duplicates';
		accessor: #removeDuplicates;
		label: 'Remove Duplicates';
		default: true;
		priority: 260;
		yourself! !

!PRVisitedWidget class methodsFor: 'testing' stamp: 'lr 8/29/2008 11:15'!
isAbstract
	^ false! !

!PRVisitedWidget class methodsFor: 'accessing' stamp: 'lr 8/29/2008 11:51'!
label
	^ 'Visited Pages'! !

!PRVisitedWidget methodsFor: 'initialization' stamp: 'lr 8/29/2008 11:15'!
initialize
	super initialize.
	structures := OrderedCollection new! !

!PRVisitedWidget methodsFor: 'accessing' stamp: 'lr 8/29/2008 11:24'!
items
	^ structures! !

!PRVisitedWidget methodsFor: 'events' stamp: 'lr 8/29/2008 11:30'!
onChangeContext: aContext
	super onChangeContext: aContext.
	(structures isEmpty or: [ structures last ~= aContext structure ]) ifTrue: [
		(self isFiltered: aContext structure) ifFalse: [ 
			self removeDuplicates
				ifTrue: [ structures remove: aContext structure ifAbsent: [ ] ].
			structures addLast: aContext structure ] ].
	self maxItems isNil ifFalse: [
		[ self maxItems < structures size ]
			whileTrue: [ structures removeFirst ] ]! !

!PRVisitedWidget methodsFor: 'accessing-settings' stamp: 'lr 8/29/2008 11:30'!
removeDuplicates
	^ self read: #descriptionRemoveDuplicates! !

!PRVisitedWidget methodsFor: 'accessing' stamp: 'lr 8/29/2008 11:58'!
unfilteredItems
	self shouldNotImplement! !

PRListWidget subclass: #PRViewsWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRViewsWidget class methodsFor: 'accessing-default' stamp: 'lr 3/17/2009 17:31'!
defaultViewClasses
	^ ((Array withAll: PRViewComponent withAllConcreteClasses)
		select: [ :each | each isListed ])
		sort: [ :a :b | a label caseInsensitiveLessOrEqual: b label ];
		yourself! !

!PRViewsWidget class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionViewClasses
	^ MAMultipleOptionDescription new
		default: [ self defaultViewClasses ] magritteDynamicObject;
		options: [ self defaultViewClasses ] magritteDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: MAClassDescription new;
		accessor: #viewClasses;
		label: 'View Classes';
		priority: 200;
		beOrdered;
		beDistinct;
		beSorted;
		yourself! !

!PRViewsWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRViewsWidget class methodsFor: 'converting' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Views'! !

!PRViewsWidget methodsFor: 'accessing' stamp: 'lr 8/15/2007 20:55'!
items
	| views |
	views := self context views.
	^ self viewClasses select: [ :each | views includes: each ]! !

!PRViewsWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:59'!
labelFor: anObject
	^ anObject label! !

!PRViewsWidget methodsFor: 'rendering' stamp: 'lr 4/18/2007 14:59'!
renderItem: aClass on: html
	html anchor
		class: 'active' if: (self isActive: aClass);
		goto: (self context view: aClass);
		with: (self labelFor: aClass)! !

!PRViewsWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
selected
	^ self context command isView
		ifTrue: [ self context command viewComponent class ]! !

!PRViewsWidget methodsFor: 'accessing' stamp: 'lr 4/18/2007 14:34'!
viewClasses
	^ self read: #descriptionViewClasses! !

PRWidget subclass: #PRMenuWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Deprecated'!

!PRMenuWidget class methodsFor: 'accessing-default' stamp: 'lr 2/14/2008 14:37'!
defaultDocument
	^ PRDocumentParser parse: '- *Home>/*
- *Syntax>/information/syntax*
- *About>/information/license*'! !

!PRMenuWidget class methodsFor: 'accessing-description' stamp: 'lr 4/4/2009 18:26'!
descriptionDocument
	^ PRDocumentDescription new
		default: self defaultDocument;
		parameterName: 'document';
		accessor: #document;
		label: 'Document';
		beRequired;
		yourself! !

!PRMenuWidget class methodsFor: 'testing' stamp: 'lr 2/14/2008 14:19'!
isAbstract
	^ false! !

!PRMenuWidget class methodsFor: 'accessing' stamp: 'lr 2/14/2008 14:19'!
label
	^ 'Menu'! !

!PRMenuWidget methodsFor: 'accessing' stamp: 'lr 5/26/2009 17:34'!
children
	^ (PROutgoingReferences on: self document) embeddedComponents 
		collect: [ :each | self context componentFor: each ]! !

!PRMenuWidget methodsFor: 'accessing' stamp: 'lr 2/14/2008 14:32'!
document
	^ self read: #descriptionDocument! !

!PRMenuWidget methodsFor: 'rendering' stamp: 'lr 8/5/2008 23:45'!
renderContentOn: html
	| visitor |
	visitor := PRMostSpecificInternalLinkVisitor new
		visit: self document.
	PRMenuRenderer new
		target: visitor target;
		continue: self document 
		in: self on: html! !

PRWidget subclass: #PRPersistencySettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets-Settings'!

!PRPersistencySettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRPersistencySettings class methodsFor: 'testing' stamp: 'lr 9/3/2008 11:59'!
isAbstract
	^ false! !

!PRPersistencySettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:25'!
label
	^ 'Persistency Settings'! !

!PRPersistencySettings methodsFor: 'actions' stamp: 'lr 9/3/2008 12:08'!
activate: aClass
	self kernel persistency class = aClass
		ifFalse: [ self kernel persistency: aClass new ]! !

!PRPersistencySettings methodsFor: 'actions' stamp: 'lr 9/4/2009 17:31'!
edit: aPersistency
	self show: (aPersistency asComponent
		addValidatedForm;
		yourself)! !

!PRPersistencySettings methodsFor: 'rendering' stamp: 'damiencassou 2/18/2009 16:03'!
renderContentOn: html
	html form: [
		html select
			labels: [ :each | each label ];
			selected: self kernel persistency class;
			list: PRPersistency withAllConcreteClasses;
			callback: [ :each | self activate: each ].
		html space.
		html submitButton
			with: 'select'.
		5 timesRepeat: [ html space ].
		html submitButton
			callback: [ self edit: self kernel persistency ];
			disabled: self kernel persistency description isEmpty;
			with: 'edit'.
		(self kernel persistency respondsTo: #snapshot) ifTrue: [
			html space.
			html submitButton
				callback: [ self kernel persistency snapshot ];
				text: 'snapshot' ] ].
	html paragraph: self kernel persistency class comment! !

PRWidget subclass: #PRSearchWidget
	instanceVariableNames: 'query'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRSearchWidget class methodsFor: 'accessing-descriptions' stamp: 'lr 9/12/2009 18:59'!
descriptionExampleText
	^ MAStringDescription new
		parameterName: 'example-button';
		accessor: #exampleText;
		label: 'Example';
		priority: 310;
		yourself! !

!PRSearchWidget class methodsFor: 'accessing-description' stamp: 'lr 9/12/2009 18:59'!
descriptionSearchButton
	^ MAStringDescription new
		parameterName: 'search-button';
		accessor: #searchButton;
		label: 'Button';
		priority: 300;
		yourself! !

!PRSearchWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier Search'! !

!PRSearchWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRSearchWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:33'!
label
	^ 'Search Field'! !

!PRSearchWidget methodsFor: 'accessing-dynamic' stamp: 'lr 9/12/2009 18:56'!
exampleText
	^ self read: #descriptionExampleText! !

!PRSearchWidget methodsFor: 'accessing' stamp: 'lr 9/12/2009 12:51'!
query
	^ query! !

!PRSearchWidget methodsFor: 'accessing' stamp: 'lr 9/12/2009 13:17'!
query: aString
	query := aString.
	context := self context 
		structure: self context structure
		view: PRSearchView.
	context command viewComponent 
		query: aString;
		hideForm.
	self context: context! !

!PRSearchWidget methodsFor: 'rendering' stamp: 'lr 9/12/2009 19:59'!
renderContentOn: html
	html form class: 'search-widget'; with: [
		html textInput
			type: 'search';
			value: self query;
			exampleText: self exampleText;
			callback: [ :value | self query: value ].
		self searchButton isEmptyOrNil
			ifFalse: [ html submitButton: self searchButton ] ]! !

!PRSearchWidget methodsFor: 'accessing-dynamic' stamp: 'lr 9/12/2009 18:56'!
searchButton
	^ self read: #descriptionSearchButton! !

PRWidget subclass: #PRTocWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRTocWidget class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRTocWidget class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Table of Contents'! !

!PRTocWidget methodsFor: 'rendering' stamp: 'lr 4/23/2007 07:44'!
renderContentOn: html
	PRTocRenderer new
		start: self context structure
		in: self on: html! !

!PRWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier Widget'! !

!PRWidget methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
context
	^ context ifNil: [ context := super context ]! !

!PRWidget methodsFor: 'private' stamp: 'lr 4/24/2008 08:28'!
expand: aString
	^ PRMacroExpander default expand: aString with: self context! !

!PRWidget methodsFor: 'accessing' stamp: 'lr 9/4/2009 17:50'!
inform: a! !

!PRWidget methodsFor: 'initialization' stamp: 'lr 5/26/2009 17:16'!
initializeOwner: aStructure link: anInternalLink
	super initializeOwner: (owner := aStructure) link: anInternalLink! !

!PRWidget methodsFor: 'accessing' stamp: 'lr 9/3/2008 12:01'!
kernel
	^ self context kernel! !

!PRWidget methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
onChangeContext: aContext
	super onChangeContext: aContext.
	context := aContext! !

!PRWidget methodsFor: 'accessing' stamp: 'lr 11/3/2007 10:00'!
owner
	"The owning structure where this widget is instantiated and configured from."
	
	^ owner! !

!PRWidget methodsFor: 'accessing-properties' stamp: 'lr 4/30/2007 21:00'!
properties
	^ properties ifNil: [ properties := Dictionary new ]! !

!PRWidget methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:30'!
propertyAt: aKey
	^ self properties at: aKey! !

!PRWidget methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:30'!
propertyAt: aKey ifAbsent: aBlock
	^ self properties at: aKey ifAbsent: aBlock! !

!PRWidget methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:30'!
propertyAt: aKey ifAbsentPut: aBlock
	^ self properties at: aKey ifAbsentPut: aBlock! !

!PRWidget methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:30'!
propertyAt: aKey put: aValue
	^ self properties at: aKey put: aValue! !

!PRWidget methodsFor: 'private' stamp: 'lr 4/18/2007 14:55'!
read: aSymbol
	^ self readUsing: (self class perform: aSymbol)! !

!PRWidget methodsFor: 'private-model' stamp: 'lr 4/30/2007 20:59'!
readUsing: aDescription
	^ self propertyAt: aDescription! !

!PRWidget methodsFor: 'rendering' stamp: 'lr 5/28/2007 16:31'!
renderContentOn: html
	"Note: If you code ends up in here you most certainly need to rename #renderWidgetOn: to #renderContentOn:"

	self subclassResponsibility! !

!PRWidget methodsFor: 'private-model' stamp: 'lr 4/18/2007 14:55'!
write: anObject using: aDescription
	^ self propertyAt: aDescription put: anObject! !

PRVisitor subclass: #PRMostSpecificInternalLinkVisitor
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRMostSpecificInternalLinkVisitor methodsFor: 'accessing' stamp: 'lr 8/5/2008 23:42'!
target
	^ target! !

!PRMostSpecificInternalLinkVisitor methodsFor: 'visiting' stamp: 'lr 8/5/2008 23:47'!
visitInternalLink: anObject
	| context structure targetStructure |
	context := PRCurrentContext value.
	structure := context structure.
	targetStructure := anObject target.
	((structure parents includes: targetStructure) 
		and: [ target isNil or: [ targetStructure parents includes: target ] ])
			ifTrue: [ target := targetStructure ]! !

PRVisitor subclass: #PRRenderer
	instanceVariableNames: 'component html'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRRenderer commentStamp: 'lr 11/13/2008 18:56' prior: 0!
The renderer is a visitor that knows the canvas and painter object it is supposed to generate HTML to.

Instance Variables
	html:		<WACanvas>
	painter:		<WAHtmlCanvas>!

PRRenderer subclass: #PRDeepRenderer
	instanceVariableNames: 'visited'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRDeepRenderer methodsFor: 'initialization' stamp: 'lr 4/19/2007 00:20'!
initialize
	super initialize.
	visited := OrderedCollection new! !

!PRDeepRenderer methodsFor: 'private' stamp: 'lr 4/7/2009 00:29'!
linkRendererFor: aLink
	"Answer an instance of the most appropriate renderer for this link. If the link is internal and embedded, avoid the embedding and simply create a reference."

	| class |
	class := aLink isBroken
		ifTrue: [ PRBrokenRenderer ]
		ifFalse: [
			(aLink isEmbedded not
				or: [ aLink isInternal
				and: [ aLink target hasDocument
				and: [ visited includes: aLink target ] ] ])
					ifTrue: [ PRReferenceRenderer ]
					ifFalse: [ PREmbeddedRenderer ] ].
	^ class parent: self! !

!PRDeepRenderer methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:30'!
start: anObject
	visited := OrderedCollection new.
	super start: anObject! !

!PRDeepRenderer methodsFor: 'public' stamp: 'lr 4/7/2009 00:02'!
structure: aStructure during: aBlock
	"Always go through this method when rendering a structure. This allows the system to remember the currently rendered structures and avoid deadly recursion as well as enables plugins to intercept when the different parts of a page are rendered."
	
	visited addLast: aStructure.
	^ aBlock ensure: [ visited removeLast ]! !

!PRDeepRenderer methodsFor: 'visiting-structure' stamp: 'lr 5/19/2009 10:58'!
visitStructure: aStructure
	self 
		structure: aStructure
		during: [ self visit: aStructure document ]! !

!PRDeepRenderer methodsFor: 'private' stamp: 'lr 5/19/2009 11:19'!
visitsOf: anObject
	^ visited occurrencesOf: anObject! !

PRDeepRenderer subclass: #PRTocRenderer
	instanceVariableNames: 'enabled current counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRTocRenderer methodsFor: 'utilities' stamp: 'lr 3/9/2006 11:30'!
enable: aBoolean while: aBlock
	| old |
	old := enabled. enabled := aBoolean.
	aBlock ensure: [ enabled := old ]! !

!PRTocRenderer methodsFor: 'utilities' stamp: 'lr 3/9/2006 11:30'!
ifEnabledDo: aBlock
	enabled ifTrue: [ aBlock value ]! !

!PRTocRenderer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
level: anInteger
	| value |
	value := anInteger min: 6 max: 1.
	value < current ifTrue: [
		anInteger + 1 to: current do: [ :index |
			counter at: index put: 0 ] ].
	counter
		at: (current := value)
		put: (counter at: value) + 1! !

!PRTocRenderer methodsFor: 'visiting' stamp: 'lr 9/24/2008 11:00'!
start: anObject
	enabled := false. current := 1.
	counter := Array new: 6 withAll: 0.
	html unorderedList
		class: 'toc';
		with: [ super start: anObject ]! !

!PRTocRenderer methodsFor: 'visiting-document' stamp: 'jf 9/30/2009 01:10'!
visitHeader: anObject 
	self level: anObject level.
	html listItem
		class: 'toc-level-' , anObject level greaseString;
		with: 
			[ html anchor
				url: '#' , anObject hash greaseString;
				with: 
					[ html span
						class: 'toc-number';
						with: 
							[ (counter first: current) do: 
								[ :each | 
								html
									render: each;
									text: '.' ] ].
					html text: ' '.
					html span
						class: 'toc-contents';
						with: 
							[ self 
								enable: true
								while: [ super visitHeader: anObject ] ] ] ]! !

!PRTocRenderer methodsFor: 'visiting-document' stamp: 'lr 3/9/2006 11:30'!
visitText: anObject
	self ifEnabledDo: [ html text: anObject text ]! !

!PRTocRenderer methodsFor: 'visiting-document' stamp: 'lr 2/2/2009 22:37'!
visitValueLink: anObject
	self ifEnabledDo: [ super visitValueLink: anObject ]! !

PRDeepRenderer subclass: #PRViewRenderer
	instanceVariableNames: 'withinContent'
	classVariableNames: 'TextEntities'
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

PRViewRenderer subclass: #PRMenuRenderer
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRMenuRenderer methodsFor: 'accessing' stamp: 'tg 4/4/2009 15:47'!
target: aStructure
	withinContent := true.
	target := aStructure! !

!PRMenuRenderer methodsFor: 'visiting-links' stamp: 'lr 4/4/2009 11:47'!
visitInternalLink: aLink
	| link |
	link := aLink copy.
	link target = target ifTrue: [
		link
			parameterAt: 'class'
			put: (aLink
				parameterAt: 'class'
				ifAbsent: [ String new ]) , ' active' ].
	super visitInternalLink: link! !

!PRViewRenderer class methodsFor: 'initialization' stamp: 'lr 5/2/2009 15:05'!
initialize
	TextEntities := OrderedCollection new.
	TextEntities
		add: '(r)' -> '&reg;';
		add: '(R)' -> '&reg;';
		add: '(c)' -> '&copy;';
		add: '(C)' -> '&copy;';
		add: '--' -> '&mdash;';
		add: '''''' -> '&rdquo;';
		add: '''' -> '&rsquo;';
		add: '``' -> '&ldquo;';
		add: '`' -> '&lsquo;'! !

!PRViewRenderer methodsFor: 'initialization' stamp: 'lr 11/13/2007 14:05'!
initialize
	super initialize.
	withinContent := false! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 5/5/2009 09:10'!
visitAnchor: anObject
	html anchor name: (anObject name
		select: [ :each | each isAlphaNumeric ])! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 5/26/2009 22:20'!
visitAnnotatedParagraph: anObject
	html paragraph
		class: 'annotated';
		class: anObject annotation;
		with: [ self visitAll: anObject children ]! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 3/30/2007 20:23'!
visitBoldFormat: anObject
	html strong: [ super visitBoldFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/30/2007 23:14'!
visitDataItem: anObject
	html definitionData: [ self visitAll: anObject children ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/30/2007 21:54'!
visitDefinitionList: anObject
	html definitionList: [ super visitDefinitionList: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/20/2007 15:11'!
visitEmptyParagraph: anObject
	"This is simple."! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 5/12/2007 20:59'!
visitHeader: anObject
	html heading level: anObject level; with: [
		html anchor name: anObject hash.
		super visitHeader: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 3/9/2006 11:30'!
visitHorizontalRule: anObject
	html horizontalRule! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 3/30/2007 20:24'!
visitItalicFormat: anObject
	html emphasis: [ super visitItalicFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/20/2007 15:10'!
visitLineBreak: anObject
	html text: ' '! !

!PRViewRenderer methodsFor: 'visiting-links' stamp: 'lr 3/24/2009 23:07'!
visitLink: anObject
	(self linkRendererFor: anObject)
		start: anObject 
		in: component
		on: html! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/9/2007 17:13'!
visitListItem: anObject
	html listItem: [ super visitListItem: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 3/30/2007 20:24'!
visitMonospaceFormat: anObject
	html code: [ super visitMonospaceFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/9/2007 17:13'!
visitOrderedList: anObject
	html orderedList: [ super visitOrderedList: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 8/30/2008 08:00'!
visitParagraph: anObject
	"Rendering paragraphs is tricky, when requiring (mostly) valid XHTML and full accessibility.	
# Paragraphs appearing within the environment are ignored alltogether.
# Other paragraphs are created using the normal XHTML paragraph markup."

	withinContent
		ifFalse: [ ^ super visitParagraph: anObject ].
	html paragraph: [ super visitParagraph: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'jf 11/2/2009 10:39'!
visitPreformatted: anObject 
	html preformatted: 
		[ anObject children 
			do: [ :each | html text: each text ]
			separatedBy: 
				[ html text: (String 
						with: Character cr
						with: Character lf) ] ]! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 12/9/2007 17:06'!
visitStrikethroughFormat: anObject
	html deleted: [ super visitStrikethroughFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 5/20/2008 08:41'!
visitSubscriptFormat: anObject
	html subscript: [ super visitSubscriptFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 5/20/2008 08:41'!
visitSuperscriptFormat: anObject
	html superscript: [ super visitSuperscriptFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/9/2007 17:12'!
visitTable: anObject
	html table: [ super visitTable: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 8/14/2008 09:03'!
visitTableCell: anObject
	(anObject isHeading ifTrue: [ html tableHeading ] ifFalse: [ html tableData ])
		style: (anObject align isNil ifFalse: [ 'text-align: ' , anObject align ]);
		with: [ super visitTableCell: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/9/2007 17:12'!
visitTableRow: anObject
	html tableRow: [ super visitTableRow: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/30/2007 23:14'!
visitTermItem: anObject
	html definitionTerm: [ self visitAll: anObject children ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 2/2/2010 15:30'!
visitText: anObject
	html text: anObject text
	
	
	"(TextEntities
		inject: anObject text
		into: [ :text :assoc | text copyReplaceAll: assoc key with: assoc value ])"! !

!PRViewRenderer methodsFor: 'visiting-formats' stamp: 'lr 12/9/2007 17:06'!
visitUnderlineFormat: anObject
	html inserted: [ super visitUnderlineFormat: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 12/9/2007 17:12'!
visitUnorderedList: anObject
	html unorderedList: [ super visitUnorderedList: anObject ]! !

!PRViewRenderer methodsFor: 'visiting-document' stamp: 'lr 2/2/2010 16:27'!
visitVerbatim: anObject
	(anObject type isNil or: [ anObject type beginsWith: 'html' ])
		ifTrue: [ html html: anObject text ]! !

!PRViewRenderer methodsFor: 'accessing' stamp: 'lr 11/13/2007 14:04'!
withinContentDo: aBlock
	"Change to the contents mode, e.g. paragraphs are no longer rendered as div but as meaningful XHTML paragraphs."
	
	| previous |
	previous := withinContent. withinContent := true.
	^ aBlock ensure: [ withinContent := previous ]! !

PRRenderer subclass: #PRLinkRenderer
	instanceVariableNames: 'parent link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

PRLinkRenderer subclass: #PRBrokenRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PRBrokenRenderer methodsFor: 'private' stamp: 'lr 3/24/2009 22:37'!
anchorFor: aLink
	^ (super anchorFor: aLink)
		class: 'broken';
		yourself! !

!PRBrokenRenderer methodsFor: 'visiting-document' stamp: 'lr 11/3/2009 21:56'!
visitInternalLink: aLink
	"Allow users to add new content, if they have the appropriate permissions."

	| lookup context |
	lookup := self context structure
		lookupIncompletePath: aLink reference.
	context := self context
		structure: lookup key
		command: PRAddCommand new.
	context command
		link: aLink;
		suggestName: lookup value.
	(self anchorFor: aLink)
		goto: context;
		with: [ self visitLink: aLink ]! !

!PRLinkRenderer class methodsFor: 'instance creation' stamp: 'lr 3/24/2009 21:31'!
parent: aVisitor
	^ self new setParent: aVisitor! !

!PRLinkRenderer methodsFor: 'private' stamp: 'lr 5/2/2009 21:41'!
anchorFor: aLink
	"Answer a default anchor for aLink."
	
	| anchor |
	anchor := html anchor.
	anchor attributes: (self attributesFor: aLink).
	anchor attributes at: 'title' ifAbsentPut: [ aLink title ].
	^ anchor! !

!PRLinkRenderer methodsFor: 'private' stamp: 'lr 3/27/2009 11:13'!
attributeNames
	"Answer a list of valid HTML attributes that can be added to links and other elements."

	^ #( 'accesskey' 'align' 'alt' 'class' 'dir' 'lang' 'disabled' 'height' 'id' 'longdesc' 'style' 'tabindex' 'title' 'width' )! !

!PRLinkRenderer methodsFor: 'private' stamp: 'lr 3/24/2009 22:48'!
attributesFor: aLink
	"Answer a set of attributes from the link."

	^ aLink parameters
		inject: WAHtmlAttributes new
		into: [ :attributes :each |
			(self attributeNames includes: each key)
				ifTrue: [ attributes at: each key put: each value ].
			attributes ]! !

!PRLinkRenderer methodsFor: 'initialization' stamp: 'lr 3/24/2009 22:29'!
setParent: aVisitor
	parent := aVisitor! !

!PRLinkRenderer methodsFor: 'visiting' stamp: 'lr 3/24/2009 23:55'!
start: aLink
	super start: (link := aLink)! !

!PRLinkRenderer methodsFor: 'visiting-document' stamp: 'lr 3/24/2009 22:40'!
visitExternalLink: aLink
	(self anchorFor: aLink)
		url: aLink url;
		class: 'external';
		with: [ self visitLink: aLink ]! !

!PRLinkRenderer methodsFor: 'visiting-document' stamp: 'lr 5/24/2009 19:37'!
visitLink: aLink
	"If present render the alias of the link with the default renderer, otherwise simply display the title."

	aLink hasAlias
		ifTrue: [ parent visitAll: aLink children ]
		ifFalse: [ html html: aLink title ]! !

PRLinkRenderer subclass: #PRReferenceRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

PRReferenceRenderer subclass: #PREmbeddedRenderer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Visitors'!

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 5/26/2009 17:34'!
visitComponent: aStructure
	parent 
		structure: aStructure
		during: [ html render: (self context componentFor: link) ]! !

!PREmbeddedRenderer methodsFor: 'visiting-document' stamp: 'jf 1/8/2010 21:39'!
visitExternalLink: aLink
	"Goes through all the pragmas and check for the first match with the given URL."
	
	| pragmas |
	pragmas := Pragma
		allNamed: #external:priority:
		from: self class 
		to: PREmbeddedRenderer
		sortedByArgument: 2.
	pragmas do: [ :pragma |
		(aLink url greaseString includesSubstring: pragma arguments first caseSensitive: false)
			ifTrue: [ ^ self perform: pragma selector with: aLink ] ].
	super visitExternalLink: aLink! !

!PREmbeddedRenderer methodsFor: 'visiting-document' stamp: 'lr 3/24/2009 23:35'!
visitExternalLinkIframe: aLink
	<external: '.htm' priority: 0>
	<external: '.html' priority: 0>

	html iframe
		attributes: (self attributesFor: aLink);
		title: aLink title;
		url: aLink url! !

!PREmbeddedRenderer methodsFor: 'visiting-document' stamp: 'lr 3/24/2009 22:56'!
visitExternalLinkImage: aLink
	<external: '.png' priority: 0>
	<external: '.jpg' priority: 0>
	<external: '.jpeg' priority: 0>
	<external: '.gif' priority: 0>

	html image
		attributes: (self attributesFor: aLink);
		title: aLink title;
		url: aLink url! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 6/8/2009 09:07'!
visitFile: aStructure
	"This code does not use the Magritte rendering to be able to add additional attributes to the generated tags. Similar to the internal embedded links it goes the pragmas and checks for the first match with the given mimetype. If no renderer can be found, simply put a link."

	| pragmas |
	aStructure file ifNil: [ ^ self ].
	parent structure: aStructure during: [
		(link hasParameter: 'anchor') ifTrue: [
			html anchor name: (aStructure name
				select: [ :each | each isAlphaNumeric ]) ].
		(link hasParameter: 'download')
			ifTrue: [ ^ self visitFileDownload: aStructure ].
		pragmas := Pragma
			allNamed: #internal:priority:
			from: self class
			to: PREmbeddedRenderer
			sortedByArgument: 2.
		pragmas do: [ :pragma |
			(aStructure file mimetype beginsWith: pragma arguments first)
				ifTrue: [ ^ self perform: pragma selector with: aStructure ] ].
		self visitFileDownload: aStructure ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 3/25/2009 00:25'!
visitFileDownload: aStructure
	<internal: 'application/' priority: 0>
	
	(self anchorFor: link)
		goto: (self context structure: aStructure view: PRDownloadView);
		with: [ self visitLink: link ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'jf 11/4/2009 10:24'!
visitFileImage: aStructure
	<internal: 'image/' priority: 0>
	
	html image
		attributes: (self attributesFor: link);
		altText: aStructure title;
		url: aStructure file url! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'jf 11/4/2009 10:25'!
visitFileMedia: aStructure
	<internal: 'video/' priority: 0>
	<internal: 'audio/' priority: 0>

	| url mime |
	url := aStructure file url.
	mime := aStructure file mimetype.
	(html tag: 'object')
		attributes: (self attributesFor: link);
		attributeAt: 'data' put: url;
		attributeAt: 'type' put: mime;
		with: [ 
			(html tag: 'param')
				attributeAt: 'name' put: 'src';
				with: url.
			(html tag: 'param')
				attributeAt: 'name' put: 'scale';
				with: 'aspect'.
			(html tag: 'param')
				attributeAt: 'name' put: 'autoplay';
				with: 'false'.
			(html tag: 'embed')
				attributes: (self attributesFor: link);
				attributeAt: 'src' put: url;
				attributeAt: 'scale' put: 'aspect';
				attributeAt: 'autoplay' put: 'false';
				attributeAt: 'type' put: mime;
				with: [ super visitInternalLink: link ] ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 1/15/2010 15:24'!
visitFileText: aStructure
	<internal: 'text/' priority: 0>

	html preformatted
		attributes: (self attributesFor: link);
		with: aStructure contents! !

!PREmbeddedRenderer methodsFor: 'visiting-document' stamp: 'lr 5/19/2009 11:19'!
visitInternalLink: aLink
	"Visits the target to embed it, if the user has permissions to see the target, ignore otherwise."

	(parent visitsOf: aLink target) > 1
		ifTrue: [ super visitInternalLink: aLink ]
		ifFalse: [
			(self context structure: aLink target) isValid
				ifTrue: [ self visit: aLink target ] ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 4/4/2009 21:50'!
visitPage: aPage
	(link hasParameter: 'menu')
		ifTrue: [ self visitPageMenu: aPage ]
		ifFalse: [ super visitPage: aPage ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 4/6/2009 23:54'!
visitPageMenu: aPage 
	| visitor |
	visitor := PRMostSpecificInternalLinkVisitor new.
	visitor visit: aPage document.
	parent structure: aPage during: [
		PRMenuRenderer new
			copyFrom: parent;
			target: visitor target;
			continue: aPage document
			in: aPage on: html ]! !

!PREmbeddedRenderer methodsFor: 'visiting-structure' stamp: 'lr 4/6/2009 23:37'!
visitStructure: aStructure
	parent visit: aStructure! !

!PRReferenceRenderer methodsFor: 'private' stamp: 'lr 6/2/2009 09:51'!
configureCommandIn: aContext link: aLink
	aContext command description do: [ :description |
		aLink
			parameterAt: description parameterName
			ifPresent: [ :value | aContext command write: (description fromString: value) using: description ] ]! !

!PRReferenceRenderer methodsFor: 'private' stamp: 'lr 6/2/2009 09:55'!
configureViewIn: aContext link: aLink
	aContext command isView
		ifFalse: [ ^ self ].
	aContext command viewComponentClass description do: [ :description |
		aLink
			parameterAt: description parameterName
			ifPresent: [ :value | aContext command viewComponent write: (description fromString: value) using: description ] ]! !

!PRReferenceRenderer methodsFor: 'private' stamp: 'lr 6/2/2009 09:50'!
contextForCommandIn: aContext link: aLink
	aLink parameterAt: 'command' ifPresent: [ :name |
		PRCommand withAllConcreteClassesDo: [ :class |
			(class name = name or: [ class label = name ]) 
				ifTrue: [ ^ aContext command: class new ] ] ].
	^ aContext! !

!PRReferenceRenderer methodsFor: 'querying' stamp: 'lr 6/2/2009 09:54'!
contextForLink: aLink
	| context |
	context := self context structure: aLink target.
	context := self contextForCommandIn: context link: aLink.
	self configureCommandIn: context link: aLink.
	context := self contextForViewIn: context link: aLink.
	self configureViewIn: context link: aLink.
	context command answer: self context.
	^ context! !

!PRReferenceRenderer methodsFor: 'private' stamp: 'lr 6/2/2009 09:51'!
contextForViewIn: aContext link: aLink
	aLink parameterAt: 'view' ifPresent: [ :name |
		PRViewComponent withAllConcreteClassesDo: [ :class |
			(class name = name or: [ class label = name ]) 
				ifTrue: [ ^ aContext view: class ] ] ].
	^ aContext! !

!PRReferenceRenderer methodsFor: 'visiting-document' stamp: 'lr 3/24/2009 22:37'!
visitExternalLink: aLink
	| anchor |
	anchor := self anchorFor: aLink.
	anchor class: 'external'; url: aLink url.
	(aLink hasCounter and: [ html callbacks isKindOf: WACallbackRegistry ]) ifTrue: [
		anchor onClick: (html request
			callback: [ aLink incrementCounter ];
			asynchronous: false;
			return: true) ].
	anchor with: [ self visitLink: aLink ]! !

!PRReferenceRenderer methodsFor: 'visiting-document' stamp: 'lr 2/23/2010 21:11'!
visitInternalLink: aLink
	| anchor |
	anchor := self anchorFor: aLink.
	anchor class: 'internal'; goto: (self contextForLink: aLink).
	aLink anchor isNil ifFalse: [ 
		anchor url fragment: (aLink anchor
			select: [ :each | each isAlphaNumeric ]) ].
	(aLink hasCounter and: [ html callbacks isKindOf: WACallbackRegistry ]) ifTrue: [
		anchor url queryFields 
			at: (html callbacks registerCallback: [ aLink incrementCounter ]) 
			put: nil ].
	anchor with: [ self visitLink: aLink ]! !

!PRReferenceRenderer methodsFor: 'visiting-document' stamp: 'lr 3/25/2009 23:35'!
visitValueLink: aLink
	"Value links are always embedded, there is no point in not embedding them. Values can be either a document (the document is visited) or any other Smalltalk object (the value is renderered). If the value is nil, the alias of the link is displayed."

	| value |
	value := aLink valueIn: self context.
	value isNil
		ifTrue: [ ^ self visitLink: aLink ].
	(value isCollection and: [ value isString not ])
		ifTrue: [ ^ parent visitAll: value ].
	(value isKindOf: PRDocumentItem)
		ifTrue: [ ^ parent visit: value ].
	html render: value! !

!PRRenderer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
component
	^ component! !

!PRRenderer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
component: aComponent
	component := aComponent! !

!PRRenderer methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:30'!
context
	^ PRCurrentContext value! !

!PRRenderer methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:30'!
context: aContext
	PRCurrentContext value: aContext! !

!PRRenderer methodsFor: 'visiting' stamp: 'lr 4/8/2009 20:21'!
continue: anObject in: aComponent on: aRenderer
	| previous |
	previous := self shallowCopy.
	self component: aComponent.
	self renderer: aRenderer.
	[ self visit: anObject ]
		ensure: [ self copyFrom: previous ]! !

!PRRenderer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
renderer
	^ html! !

!PRRenderer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
renderer: aRenderer
	html := aRenderer! !

!PRRenderer methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:30'!
start: anObject in: aComponent on: aRenderer
	self component: aComponent.
	self renderer: aRenderer.
	self start: anObject! !

!PRVisitor methodsFor: '*pier-seaside-visiting-structure' stamp: 'lr 3/9/2006 11:30'!
visitComponent: anObject
	self visitStructure: anObject! !

!PRContext methodsFor: '*pier-seaside-private' stamp: 'jf 11/4/2009 12:47'!
activationCallback
	^ WAActionCallback on: [
			PRCurrentContext value: self.
			(self command isView and: [ self command viewComponent isFullResponse ])
				ifTrue: [ self command viewComponent handle: self requestContext ] ]! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'jf 9/14/2009 16:38'!
application
	"Answer the cached seaside application, this is a short-cut to avoid to unnecessary lookups."

	^ self propertyAt: #application ifAbsentPut: [ 
		(self requestContextIfNilDo: [ ^ nil ]) application ]! !

!PRContext methodsFor: '*pier-seaside-private' stamp: 'lr 5/26/2009 17:30'!
buildComponent: aStructure for: aLink
	^ (aStructure componentFor: aLink)
		onChangeContext: self;
		yourself! !

!PRContext methodsFor: '*pier-seaside-private' stamp: 'lr 5/26/2009 17:33'!
componentDictionary
	^ self propertyAt: #components ifAbsentPut: [ IdentityDictionary new ]! !

!PRContext methodsFor: '*pier-seaside-caching' stamp: 'lr 5/26/2009 17:34'!
componentFor: aLink
	"Return the cached component of aLink, if none is existing yet create a new one."

	^ self componentDictionary at: aLink ifAbsentPut: [ self buildComponent: aLink target for: aLink ]! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
components
	^ self componentDictionary values! !

!PRContext methodsFor: '*pier-seaside-caching' stamp: 'lr 5/26/2009 17:37'!
flushComponent: aStructure
	"Flush all the components that are instantiated from aStructure."
	
	self componentDictionary keys do: [ :each |
		each target = aStructure 
			ifTrue: [ self componentDictionary removeKey: each ifAbsent: [ ] ] ]! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'jf 9/14/2009 16:05'!
requestContext
	^ WACurrentRequestContext value! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'jf 10/1/2009 12:13'!
requestContextIfNilDo: aBlock
	^ [ WACurrentRequestContext value ]
		on: WARequestContextNotFound
		do: [ :e | aBlock value ]! !

!PRContext methodsFor: '*pier-seaside-private' stamp: 'jf 11/4/2009 12:37'!
sanitizeSeasideCache
	<sanitize>
	
	self properties
		removeKey: #components ifAbsent: [ ];
		removeKey: #session ifAbsent: [ ];
		removeKey: #application ifAbsent: [ ]! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'jf 9/14/2009 16:38'!
session
	"Answer the cached seaside session, this is a short-cut to avoid to unnecessary lookups."

	^ self propertyAt: #session ifAbsentPut: [
		(self requestContextIfNilDo: [ ^ nil ]) session ]! !

!PRContext methodsFor: '*pier-seaside-navigation' stamp: 'lr 4/10/2007 21:47'!
structure: aStructure view: aViewComponentClass
	"Create a copy of the current context with the current structure replaced by ==aStructure== and the current view by ==aViewComponent==."

	^ self structure: aStructure command: (aStructure viewCommandClass new
		viewComponentClass: aViewComponentClass;
		yourself)! !

!PRContext methodsFor: '*pier-seaside-urls' stamp: 'NickAger 1/13/2010 12:05'!
url
	"Answer a REST URL to get into the receiving context."
	
	^ self application url
		takeServerParametersFromRequest: self requestContext request;
		adaptToContext: self;
		yourself! !

!PRContext methodsFor: '*pier-seaside-urls' stamp: 'lr 2/23/2010 21:13'!
urlOn: aRenderer
	"Answer a Seaside URL to get into the receiving context."

	| url |
	^ aRenderer callbacks isNil
		ifTrue: [ self url ]
		ifFalse: [
			url := aRenderer context actionUrl copy.
			url purgeOtherFields; adaptToContext: self.
			(self command isRestful and: [ self application hasCookieInContext: self requestContext ])
				ifTrue: [ url purgeSeasideFields ]			
				ifFalse: [
					url queryFields
						at: '_n' put: nil;
						at: (aRenderer callbacks store: self activationCallback) put: nil ].
			url ]! !

!PRContext methodsFor: '*pier-seaside-navigation' stamp: 'lr 4/10/2007 21:47'!
view: aViewComponentClass
	"Create a copy of the current context ==aViewComponent==."

	^ self structure: self structure view: aViewComponentClass! !

!PRContext methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
views
	"Answer a list of possible views, dispatching trough the command class."

	^ PRViewComponent withAllConcreteClasses
		select: [ :each | each isValidIn: self ]! !

!PRDiffAlgorithm methodsFor: '*pier-seaside-rendering' stamp: 'lr 9/28/2009 19:58'!
renderOn: html
	"Display a change list on the html canvas."

	self patchsetDo: [ :operation :object :index |
		operation = #+
			ifTrue: [ html inserted: object ]
			ifFalse: [
				operation = #-
					ifTrue: [ html deleted: object ]
					ifFalse: [ html render: object ] ].
		html break ]! !

!WAComponent class methodsFor: '*pier-seaside' stamp: 'lr 9/3/2008 11:49'!
groupLabel
	^ 'Seaside Component'! !

!WAComponent class methodsFor: '*pier-seaside' stamp: 'lr 5/26/2009 17:18'!
owner: aStructure link: anInternalLink
	^ self new initializeOwner: aStructure link: anInternalLink! !

!WAComponent methodsFor: '*pier-seaside-initialization' stamp: 'lr 2/20/2010 12:43'!
initializeAttribute: description with: aValue overridWith: anInternalLink
	"Whenever a component is instantiated it is initialized with the settings from the model, that
	can be overridden with the settings given in the link that embeds the component."

	| preference |
	preference := aValue.
	anInternalLink notNil ifTrue: [
		anInternalLink 
			parameterAt: description parameterName 
			ifPresent: [ :value | preference := description fromString: value ] ].
	self
		write: (preference ifNil: [ description default yourself ])
		using: description ! !

!WAComponent methodsFor: '*pier-seaside-initialization' stamp: 'razavi@acm.org 2/18/2010 07:22'!
initializeOwner: aStructure link: anInternalLink
	"Initialize the receiver with the settings from aStructure and overriding parameters of anInternalLink."

	aStructure 
		initializeVisualComponentSettings: self 
		link: anInternalLink! !

WASystemConfiguration subclass: #PRPierConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Core'!

!PRPierConfiguration methodsFor: 'description' stamp: 'jf 11/3/2009 12:59'!
describeOn: config
	(config list: #kernel)
		group: #pier;
		options: [ PRKernel instances ].
	
	config
		at: #rootClass put: PRPierFrame! !

!PRPierConfiguration methodsFor: 'accessing' stamp: 'jf 9/6/2009 20:03'!
parents
	^ Array with: WARenderLoopConfiguration instance! !

MATextAreaComponent subclass: #PRDocumentComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Magritte'!

!PRDocumentComponent methodsFor: 'rendering' stamp: 'lr 9/9/2009 19:11'!
renderViewerOn: html
	html rendererInstance visit: self value! !

!PRAddCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 8/11/2007 21:57'!
saveButton
	^ super saveButton value: self class label! !

!GRPackage class methodsFor: '*pier-seaside' stamp: 'lr 2/18/2010 10:34'!
pierSeaside
	^ self new
		name: 'Pier-Seaside';
		description: 'Seaside integration of the Pier content management system.';
		addDependency: 'Magritte-Seaside';
		addDependency: 'Pier-Model';
		url: #pierUrl;
		yourself! !

!MAContainerComponent methodsFor: '*pier-seaside-actions' stamp: 'lr 11/6/2007 13:43'!
apply
	"This is an ugly hack that will only work in the context of Pier. It assumes that the underlying model is a command and that it can be told to redisplay itself."
	
	self validate ifFalse: [ ^ self ].
	self commit.
	self model answer isNil
		ifTrue: [ self model answer: self model context copy ].
	self answer: self model! !

!PROutgoingReferences methodsFor: '*pier-seaside-accessing' stamp: 'lr 6/3/2009 11:26'!
embeddedComponents
	^ self select: [ :each | each isInternal and: [ each isEmbedded and: [ each isBroken not and: [ each target isComponent ] ] ] ]! !

!WAUrl methodsFor: '*pier-seaside-mapping' stamp: 'NickAger 1/13/2010 12:04'!
adaptToContext: aPierContext
	path := aPierContext application url path copy.
	aPierContext structure parents allButFirst
		do: [ :each | self path add: each name ].
	aPierContext command updateUrl: self! !

!WAUrl methodsFor: '*pier-seaside-mapping' stamp: 'lr 2/23/2010 20:56'!
purgeOtherFields
	self queryFields keys do: [ :each |
		(self isSeasideField: each)
			ifFalse: [ self queryFields removeKey: each ifAbsent: [ ] ] ]! !

!WAUrl methodsFor: '*pier-seaside-mapping' stamp: 'lr 2/23/2010 20:56'!
purgeSeasideFields
	self queryFields keys do: [ :each |
		(self isSeasideField: each)
			ifTrue: [ self queryFields removeKey: each ifAbsent: [ ] ] ]! !

PRStructure subclass: #PRComponent
	instanceVariableNames: 'componentClass settings link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Model'!

!PRComponent class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:48'!
componentClasses
	| sortBlock |
	sortBlock := [ :a :b | 
		a groupLabel = b groupLabel
			ifTrue: [ a label < b label ]
			ifFalse: [ a groupLabel < b groupLabel ] ].
	^ ((PRWidget allSubclasses reject: [ :each | each isAbstract ]) asArray 
		sort: sortBlock;
		yourself) ,
	  ((WAComponent allSubclasses select: [ :each | each canBeRoot ]) asArray
		sort: sortBlock;
		yourself)! !

!PRComponent class methodsFor: 'accessing-description' stamp: 'NickAger 1/27/2010 17:20'!
descriptionComponentClass
	^ MASingleOptionDescription new
		options: [ self componentClasses ] magritteDynamicObject;
		reference: MAClassDescription new;
		accessor: #componentClass;
		label: 'Component Class';
		groupBy: #groupLabel;
		priority: 200;
		beRequired;
		beEditable;
		yourself! !

!PRComponent class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
isAbstract
	^ false! !

!PRComponent class methodsFor: 'accessing' stamp: 'lr 1/8/2010 22:15'!
label
	^ 'Component'! !

!PRComponent methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:30'!
accept: aVisitor
	aVisitor visitComponent: self! !

!PRComponent methodsFor: 'accessing-dynamic' stamp: 'lr 5/26/2009 17:22'!
component
	"Answer a new instance of the Seaside component."

	^ self componentFor: self document! !

!PRComponent methodsFor: 'accessing' stamp: 'lr 5/26/2009 17:24'!
componentClass
	"Answer the component class of the receiver. The default is an empty Seaside component to avoid any troubles."

	^ (componentClass isNil or: [ componentClass isObsolete ])
		ifFalse: [ componentClass ]
		ifTrue: [ WAComponent ]! !

!PRComponent methodsFor: 'accessing' stamp: 'lr 8/17/2007 10:45'!
componentClass: aComponentClass
	componentClass = aComponentClass ifTrue: [ ^ self ].
	componentClass := aComponentClass.
	self flush; reset! !

!PRComponent methodsFor: 'accessing-description' stamp: 'lr 4/9/2009 08:34'!
componentDescription
	"Answer the description of the component. Directly call the description builder, in case this is a stupid Seaside comment that returns a string as its description."
	
	| description |
	description := self componentClass description.
	description isDescription
		ifFalse: [ description := MADescriptionBuilder for: self componentClass ].
	description := description collect: [ :each | each copy ].
	description do: [ :each |
		each group isNil
			ifTrue: [ each group: 'Component' ] ].
	^ description! !

!PRComponent methodsFor: 'accessing-dynamic' stamp: 'lr 5/26/2009 17:23'!
componentFor: anInternalLink
	"Answer a new instance of the Seaside component."

	^ self componentClass 
		owner: self link: anInternalLink! !

!PRComponent methodsFor: 'accessing-description' stamp: 'lr 4/9/2009 08:26'!
description
	^ (MAContainer withAll: super description) 
		addAll: self componentDescription;
		yourself! !

!PRComponent methodsFor: 'accessing' stamp: 'lr 5/26/2009 17:21'!
document
	^ link ifNil: [ link := super document ]! !

!PRComponent methodsFor: 'accessing-description' stamp: 'lr 4/9/2009 08:30'!
editDescription
	^ self hasComponent
		ifFalse: [ super editDescription ]
		ifTrue: [
			MAContainer new
				addAll: ((self basicDescription
					copyWithout: self class descriptionComponentClass)
					select: [ :each | each isEditable ]);
				add: (self class descriptionComponentClass
					beReadonly;
					yourself);
				addAll: self componentDescription;
				yourself ]! !

!PRComponent methodsFor: 'accessing-dynamic' stamp: 'lr 5/26/2009 17:32'!
embeddedComponents
	^ super embeddedComponents copyWith: self document! !

!PRComponent methodsFor: 'actions' stamp: 'jf 9/24/2009 14:33'!
flush
	| context |
	context := PRCurrentContext value.
	context isNil ifFalse: [ context flushComponent: self ]! !

!PRComponent methodsFor: 'testing' stamp: 'lr 4/4/2009 17:36'!
hasComponent
	^ componentClass notNil! !

!PRComponent methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:30'!
initialize
	super initialize.
	self reset! !

!PRComponent methodsFor: 'accessing' stamp: 'razavi@acm.org 2/18/2010 07:22'!
initializeVisualComponentSettings: aComponent link: anInternalLink
	"the components (or widgets) cannot carry their own configuration, because they are 	instantiated later on and on a per-session based. So the configuration happens on a 
	generic model object (PRComponent) that takes the description of the component and
	stores its values in the settings dictionary. Whenever a component is instantiated it is 	initialized with the settings from the model, that can be overridden with the settings
	given in the link that embeds the component."
	
	 | aValue |
	self componentDescription do: [ :description |
		aValue := self visualComponentPreferenceFor: description.
		aComponent initializeAttribute: description with: aValue overridWith: anInternalLink].
! !

!PRComponent methodsFor: 'testing' stamp: 'lr 5/12/2007 16:27'!
isComponent
	^ true! !

!PRComponent methodsFor: 'copying' stamp: 'lr 8/17/2007 08:01'!
postCopy
	super postCopy.
	settings := settings copy! !

!PRComponent methodsFor: 'private' stamp: 'lr 2/18/2009 15:03'!
readUsing: aDescription
	^ (self basicDescription includes: aDescription)
		ifTrue: [ super readUsing: aDescription ]
		ifFalse: [ self settings at: aDescription ifAbsentPut: [ aDescription default yourself ] ]! !

!PRComponent methodsFor: 'actions' stamp: 'lr 5/22/2007 23:57'!
reset
	settings := Dictionary new! !

!PRComponent methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
settings
	^ settings! !

!PRComponent methodsFor: 'accessing-description' stamp: 'lr 4/9/2009 08:32'!
settingsDescription
	^ self hasComponent
		ifFalse: [ super settingsDescription ]
		ifTrue: [ 
			MAContainer new
				add: self class descriptionComponentClass;
				addAll: (self basicDescription
					select: [ :each | each isSetting ]);
				yourself ]! !

!PRComponent methodsFor: 'accessing' stamp: 'razavi@acm.org 2/18/2010 07:09'!
visualComponentPreferenceFor: aDescription
	^self settings at: aDescription ifAbsent: []! !

!PRComponent methodsFor: 'private' stamp: 'lr 2/18/2009 15:03'!
write: anObject using: aDescription
	(self basicDescription includes: aDescription)
		ifTrue: [ super write: anObject using: aDescription ]
		ifFalse: [
			(anObject notNil and: [ aDescription isDocumentDescription ])
				ifTrue: [ anObject owner: self ].
			self settings at: aDescription put: anObject ]! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 2/5/2009 21:35'!
descriptionEnvironment
	^ PRStructureDescription new
		accessor: #localEnvironment;
		addCondition: (MACondition selector: #hasDocument)
			labelled: 'The environment has to have a document';
		addCondition: (MACondition selector: #hasContentsWidget)
			labelled: 'Invalid environment, make sure to embed the contents widget';
		group: 'Layout Settings';
		label: 'Environment';
		priority: 800;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 8/24/2008 10:21'!
descriptionHideFromMenus
	^ MABooleanDescription new
		comment: 'Hides this structure from navigation facilities.';
		checkboxLabel: 'Hide from menus';
		accessor: #hideFromMenus;
		label: 'Visibility';
		priority: 10;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 1/11/2010 21:27'!
descriptionIcon
	^ PRStructureDescription new
		accessor: #localIcon;
		addCondition: (MACondition selector: #isFile) & (MACondition selector: #isImage)
			labelled: 'Invalid icon';
		group: 'Layout Settings';
		label: 'Shortcut Icon';
		priority: 820;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 3/26/2009 20:58'!
descriptionNavigationTitle
	^ MAStringDescription new
		comment: 'A short title used for navigation facilities.';
		label: 'Navigation Title';
		parameterName: 'menu';
		accessor: #menu;
		beSearchable;
		priority: 20;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 2/5/2009 21:37'!
descriptionStyleSheet
	^ PRStructureDescription new
		addCondition: (MACondition selector: #isFile) & (MACondition selector: #isText)
			labelled: 'Invalid style sheet';
		accessor: #localStyleSheet;
		group: 'Layout Settings';
		label: 'Style Sheet';
		priority: 810;
		beSetting;
		yourself! !

!PRStructure class methodsFor: '*pier-seaside-descriptions' stamp: 'lr 7/2/2009 08:12'!
descriptionTagsSeaside: aDescription
	^ aDescription beSetting! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 5/26/2009 17:34'!
componentsIn: aContext
	^ self embeddedComponents collect: [ :each | aContext componentFor: each ]! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 8/10/2007 18:53'!
cssClass
	"Answer a CSS class that should be used in the body-tag when browsing the structure. Furthermore it is put into menus and links that reference the receiver."

	^ self class fullName asLowercase allButFirst: 2! !

!PRStructure methodsFor: '*pier-seaside-configuration' stamp: 'lr 2/2/2010 17:43'!
defaultEnvironment
	^ (PRPage named: 'environment')
		addChild: ((PRFile named: 'style.css')
			filename: 'style.css';
			mimetype: 'text/css';
			contents: '/* blueprint patches */
h1, h2, h3, h4, h5, h6 { margin: 0; font-weight: inherit; font-size: 100%; line-height: 1; }
h1 { font-size: 2em; margin-bottom: 0.75em; }
h2 { font-size: 1.5em; line-height: 1; margin-bottom: 1em; }
h3 { font-size: 1.2em; height: 1.25em; line-height: 1.25; margin-bottom: 1.25em; }
h4 { font-size: 1em; font-weight: bold; margin-bottom: 1.5em; }
h5 { font-size: 1em; font-weight: bold; }
h6 { font-size: 1em; }

/* pier */
a.broken { color: #8a1f11; }
a.protected { color: #514721; }
h1.heading { font-size: 3em; line-height: 1; margin-bottom: 0.5em; }
pre, code, tt { font-size: 1.3em; background-color: #eaeaea; }

/* widgets */
ul.toc { list-style: none; margin-left: 0; }
ul.toc li.toc-level-1 { margin-left: 0em; }
ul.toc li.toc-level-2 { margin-left: 1em; }
ul.toc li.toc-level-3 { margin-left: 2em; }
ul.toc li.toc-level-4 { margin-left: 3em; }
ul.toc li.toc-level-5 { margin-left: 4em; }
ul.toc li.toc-level-6 { margin-left: 5em; }

/* magritte */
.magritte td { vertical-align: top; }
.magritte textarea { width: 100%; height: 250px; padding: 0; margin: 0; }
.magritte tr td.required:after { content: "*"; color: #8a1f11; font-weight: bold; margin-left: 0.5em; }
.magritte th { vertical-align: top; padding-top: 10px; background-color: inherit; width: 120px; }
.magritte tr.group th { font-size: 1.5em; line-height: 1; margin-bottom: 1em; width: 100% }
.magritte .error { border: 0; padding: 4px 10px 4px 5px; }

/* default look */
.header .title a.page {
	height: 59px;
	outline: none;
	display: block;
	text-indent: -1000em;
	background: url(/pier/environment/pier.png?view=PRDownloadView) no-repeat;
}
.header .search input {
	width: 140px;
	outline: none;
}
.header .search ul {
	margin: 0;
	padding: 0;
	background: #fff;
	list-style-type: none;
	border: 1px solid #666;
	border-top: none;
}
.header .search li.selected {
	background-color: #009;
}
.header .search li {
	margin: 0;
	display: block;
	padding: 0.2em;
	cursor: pointer;
	list-style-type: none;
}
.header .search a {
	text-decoration: none;
}
.header .search li.selected a {
	color: #fff;
}
.header .children {
	height: 4em;
}
.header .children ul {
	margin: 0 0 1em 0;
	list-style-type: none;
}
.header .children ul li {
	float: left;
	width: 110px;
	margin-right: 10px;
	list-style-type: none;
}
.header .children .active {
	font-weight: bold;
}
.sidebar ul {
	margin-top: 0;
	margin-left: 0;
	list-style-type: none;
}
.sidebar ul ul {
	margin-bottom: 0;
	margin-left: 1.5em;
}
.sidebar .active {
	font-weight: bold;
}
.footer {
	font-size: 0.8em;
	margin-top: 1em;
	text-align: center;
}';
			yourself);
		addChild: ((PRFile named: 'pier.png')
			filename: 'pier.png';
			mimetype: 'image/png';
			contents: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 125 0 0 0 59 8 2 0 0 0 217 227 18 3 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 11 128 73 68 65 84 120 218 236 92 95 108 20 199 25 159 221 251 135 237 59 167 150 33 22 118 8 146 35 25 91 10 180 17 9 8 94 138 2 60 244 1 106 250 84 12 177 213 23 136 68 250 2 168 41 72 141 44 30 74 26 21 164 170 32 5 75 81 98 2 230 165 170 221 152 170 145 48 200 17 18 22 6 75 169 105 226 179 19 211 144 96 163 68 118 0 223 249 236 251 59 253 237 140 111 188 183 123 183 55 187 103 83 27 238 211 228 56 150 221 157 217 223 126 243 251 126 223 55 115 81 40 165 100 233 89 98 42 52 121 245 218 163 254 219 211 193 225 240 80 80 28 119 151 7 42 183 191 94 211 188 207 223 80 79 150 179 41 75 13 247 71 253 183 198 206 95 156 236 185 102 125 218 134 243 31 252 100 211 107 69 220 23 6 241 111 207 188 143 79 153 147 87 212 84 111 186 250 233 242 197 221 189 68 198 49 122 242 189 177 246 11 242 231 207 142 141 47 107 158 89 18 184 15 31 251 195 247 157 255 176 117 9 252 125 89 227 174 46 5 122 177 11 58 172 106 207 47 139 184 23 100 136 162 118 47 129 152 169 105 222 95 196 189 32 75 78 133 236 130 190 161 253 3 8 202 34 238 79 206 106 90 246 63 5 160 47 137 184 138 36 72 70 59 86 238 120 125 237 161 55 151 123 186 180 180 244 59 178 164 123 103 223 215 231 165 122 86 169 106 220 13 208 151 187 128 89 186 121 19 36 249 236 216 216 227 254 219 218 52 12 4 202 26 214 249 235 235 159 2 74 89 30 117 130 121 157 51 254 253 205 129 65 124 246 15 220 105 88 87 91 179 186 106 199 182 45 53 213 85 207 28 238 67 195 119 241 9 8 100 206 12 133 195 248 18 240 251 101 206 215 91 255 192 96 79 111 31 62 121 119 220 214 164 226 165 52 53 161 186 95 122 245 149 119 91 15 23 136 62 110 62 21 154 30 26 30 157 143 49 213 85 47 84 87 109 218 184 193 174 79 224 251 230 87 55 100 189 16 255 138 7 153 10 105 56 148 7 252 6 167 201 131 59 198 215 211 123 227 106 111 31 110 193 143 0 199 243 231 222 43 15 148 101 237 166 243 242 21 61 94 172 203 178 230 189 141 45 77 123 204 151 24 122 185 53 112 7 159 248 142 35 235 146 209 117 169 88 93 50 202 17 159 207 108 93 190 182 85 107 79 182 30 193 99 20 254 58 205 246 171 93 59 223 58 176 47 215 123 197 216 218 59 58 59 47 247 112 196 133 157 61 245 142 126 60 56 237 76 219 133 246 142 46 243 205 143 29 57 200 113 200 137 59 110 125 166 237 162 0 194 112 253 201 214 195 121 187 201 16 45 213 85 103 255 252 142 193 247 1 129 246 82 63 235 19 88 108 77 68 94 73 204 212 165 98 122 172 13 246 161 175 226 63 21 171 240 238 101 102 18 158 226 239 221 87 204 72 89 219 241 35 7 91 154 26 13 247 105 191 212 213 217 125 197 140 6 247 197 174 142 179 226 175 111 28 120 27 47 56 235 157 133 215 102 193 29 215 0 68 225 224 102 195 180 250 184 237 79 252 59 78 59 214 122 42 235 104 12 134 206 48 56 188 0 220 191 179 187 71 255 70 43 105 114 119 108 234 103 201 217 50 154 242 16 226 98 7 83 186 166 31 98 183 183 252 19 79 0 247 185 214 253 145 53 226 240 27 128 238 140 139 4 244 218 107 235 238 201 133 163 176 91 189 127 227 142 252 215 115 23 208 175 197 153 28 189 12 253 142 62 206 95 234 178 158 137 130 31 241 37 175 155 27 38 233 239 91 79 227 133 235 47 1 226 191 137 62 4 171 248 32 25 9 241 165 143 211 52 226 137 204 70 51 29 25 51 47 107 95 120 120 60 136 140 55 228 50 60 26 62 225 227 146 19 37 56 50 10 64 209 35 250 205 139 30 70 238 118 54 208 198 166 67 161 80 216 238 131 161 75 189 227 128 85 0 58 224 94 69 136 23 148 199 154 1 247 36 131 59 206 90 140 16 125 127 136 7 102 220 1 211 161 163 39 242 186 142 140 151 252 241 212 57 249 243 111 222 30 4 238 96 127 25 76 16 47 149 199 83 225 230 131 191 43 124 160 118 13 62 126 116 118 162 134 144 10 17 226 51 79 160 233 79 202 208 231 184 255 64 200 199 222 231 122 60 126 78 92 152 221 250 75 228 73 111 193 13 131 217 179 107 167 252 236 119 17 111 229 245 27 3 79 126 160 191 142 61 126 153 38 158 79 187 185 194 255 80 231 221 30 71 196 49 52 15 107 152 243 159 122 2 147 170 54 77 163 177 248 142 109 91 87 173 172 16 36 121 248 248 187 56 104 45 87 160 133 98 177 248 208 200 2 251 25 250 253 247 157 160 141 186 152 227 200 83 160 149 16 234 226 160 186 137 226 101 205 151 254 194 155 71 107 90 1 73 157 123 7 156 19 35 138 170 115 240 27 2 244 99 173 167 173 123 228 50 12 1 6 74 241 217 173 71 206 16 5 25 69 196 77 102 221 140 221 121 243 105 232 19 93 19 239 0 51 115 154 205 131 239 84 207 60 81 126 214 151 166 151 211 121 121 0 218 89 136 218 255 63 238 242 73 154 133 48 114 112 213 39 222 242 160 226 25 141 145 175 102 201 189 40 115 118 134 50 49 249 62 73 31 76 185 200 164 226 50 100 0 8 212 224 244 188 221 33 119 19 137 219 147 15 102 102 39 112 35 215 194 108 133 12 24 127 240 67 214 44 201 250 122 158 191 224 73 160 112 108 245 13 183 61 81 2 122 39 72 145 254 18 121 48 171 146 18 111 102 84 165 132 50 89 163 112 89 147 36 51 73 50 65 141 133 235 67 71 78 200 140 89 175 124 242 138 113 110 32 37 92 133 155 227 209 108 165 93 128 197 122 72 240 84 55 78 18 99 170 239 168 181 37 158 222 58 176 159 39 141 248 196 189 36 159 199 96 156 175 147 174 180 150 212 225 174 164 19 39 202 112 159 157 33 35 46 159 89 240 229 237 66 43 171 233 184 165 253 146 148 234 64 208 230 32 214 172 174 146 199 29 193 3 23 90 123 33 50 50 213 48 62 121 188 240 36 250 100 122 211 198 245 206 38 93 165 6 42 241 149 166 185 37 51 186 10 146 65 139 81 50 145 201 51 14 152 16 17 88 6 68 248 162 69 65 201 250 109 1 70 67 153 193 48 24 52 213 49 83 183 236 109 92 16 178 91 147 138 187 160 102 203 116 160 235 162 171 32 125 180 153 184 145 223 37 173 190 174 86 76 142 147 18 19 90 31 132 237 26 247 93 193 4 230 127 5 177 23 164 103 12 21 193 254 129 59 206 238 243 98 42 94 170 57 187 66 50 155 226 229 109 206 223 227 38 49 35 111 33 198 69 0 29 25 162 12 47 65 227 59 115 118 193 102 60 242 233 131 10 215 175 93 29 103 249 157 221 134 36 91 190 131 133 82 99 107 146 177 18 191 6 241 156 15 40 233 36 21 113 53 69 73 82 81 52 114 167 177 25 45 18 232 197 187 188 117 94 190 130 209 158 105 187 32 163 100 16 78 77 46 37 27 183 16 9 244 147 6 183 210 23 110 245 150 137 251 3 89 220 55 155 24 201 89 80 229 9 148 175 140 249 184 194 178 86 30 86 117 74 134 178 26 77 120 150 58 115 118 46 28 33 123 228 53 140 221 184 237 192 28 238 39 16 140 105 119 162 100 45 212 148 86 184 230 252 61 141 187 94 201 40 9 40 27 37 73 29 146 187 188 248 203 186 156 18 28 25 181 19 192 215 59 193 93 62 161 48 4 141 155 78 157 157 175 111 248 202 25 161 43 233 226 204 28 201 104 206 14 146 161 172 64 51 51 67 121 89 102 49 12 130 34 215 242 225 34 37 89 238 204 57 21 118 150 163 6 157 150 153 32 102 52 220 159 83 136 75 248 59 3 94 144 76 130 147 15 141 76 211 97 183 119 49 16 255 237 193 125 22 66 206 214 163 149 7 252 139 200 51 230 88 111 177 56 149 23 247 192 42 149 64 183 112 220 85 134 123 186 248 171 145 12 254 198 200 39 153 32 17 143 186 128 112 131 19 64 229 121 213 193 144 29 158 145 79 128 220 14 180 96 125 221 75 6 114 119 204 239 165 132 186 53 133 206 252 221 69 20 94 120 20 36 147 208 190 226 208 212 36 117 44 34 133 30 127 129 65 12 201 145 11 107 120 207 208 240 168 97 127 192 147 224 25 103 38 153 118 243 36 16 238 163 127 146 58 4 213 149 42 241 40 138 91 195 157 168 10 47 18 104 36 147 160 148 113 190 6 189 90 80 80 109 222 219 152 107 69 80 36 177 103 218 46 114 239 193 23 161 106 236 138 52 189 142 204 83 143 204 200 47 194 97 185 73 154 17 181 59 165 43 248 72 28 182 255 60 67 48 172 76 37 128 187 6 186 103 46 75 66 2 165 176 164 73 59 226 81 136 91 163 32 215 10 165 146 38 183 38 34 218 254 142 100 212 46 238 188 92 156 213 128 236 27 7 222 62 214 122 90 63 101 69 78 107 215 217 229 115 26 181 192 216 221 222 33 187 42 203 201 148 23 155 244 197 25 119 137 162 45 125 128 66 188 76 213 64 200 11 208 93 132 183 210 10 165 186 193 245 166 235 209 209 217 9 180 221 113 123 91 183 241 92 0 87 255 116 124 89 188 177 233 80 214 61 23 120 34 126 48 56 178 88 21 99 39 60 19 10 79 139 231 225 235 238 50 12 203 87 121 16 121 68 153 148 139 153 242 26 149 145 59 163 26 143 182 190 167 105 71 38 105 52 198 87 231 116 100 77 131 90 93 171 144 40 29 255 58 85 247 93 148 120 236 237 155 4 142 64 89 75 179 87 87 65 146 231 245 21 78 241 247 11 200 75 100 113 151 143 141 32 22 182 103 33 140 249 40 233 236 96 88 49 7 225 242 124 113 17 206 238 11 164 211 84 149 185 54 99 21 26 103 178 93 200 121 253 70 3 133 68 194 206 3 172 164 4 16 193 192 113 18 110 7 119 233 34 1 176 150 76 187 117 26 121 254 103 49 175 109 92 207 113 127 49 21 247 249 149 140 141 3 72 80 21 58 183 226 33 140 255 19 251 111 98 60 245 104 130 246 148 250 201 162 25 102 36 47 70 218 21 105 182 170 232 139 190 190 42 42 159 186 66 102 6 197 207 169 70 32 158 160 36 78 193 36 218 103 18 60 147 110 148 55 50 121 63 245 205 23 169 15 125 21 139 87 48 208 239 254 148 119 196 185 107 51 229 181 44 238 55 111 47 252 156 226 219 34 13 121 86 185 182 181 189 118 222 151 147 12 101 13 116 66 99 148 55 77 68 38 88 234 196 94 73 52 76 131 215 19 95 12 82 128 126 195 93 186 72 160 235 235 180 206 46 95 18 254 206 119 107 102 149 86 205 108 205 228 91 213 51 253 99 42 49 163 57 56 141 51 184 163 172 105 208 19 126 240 225 127 147 95 245 198 7 255 25 191 254 208 119 162 228 121 61 232 0 168 165 169 209 238 62 239 92 132 126 173 251 35 61 25 58 0 221 86 97 220 109 86 41 11 98 64 228 120 238 37 27 60 231 249 75 93 159 15 223 253 50 238 243 253 43 182 118 179 135 169 26 38 93 224 229 81 58 53 150 2 226 63 126 147 74 198 40 176 238 41 241 139 88 202 196 232 22 196 12 94 59 148 217 141 108 161 178 246 236 218 217 162 139 249 14 146 32 158 12 91 231 101 102 155 223 15 108 177 123 216 174 239 200 188 124 190 250 115 47 248 245 174 120 104 71 60 236 242 42 101 149 154 106 153 158 164 192 58 162 168 159 187 86 12 187 124 248 228 107 29 2 235 172 119 182 222 39 109 70 10 183 218 190 109 75 94 176 16 255 45 52 27 31 207 30 71 43 177 78 112 135 35 163 51 156 124 181 183 79 40 92 204 247 250 186 90 196 76 249 65 112 93 132 251 148 210 84 93 42 6 121 19 33 10 252 122 66 117 243 200 105 112 109 201 20 9 55 68 72 52 231 128 252 199 58 184 155 45 106 226 191 137 16 218 166 60 160 253 130 197 193 239 88 114 226 190 110 227 47 36 175 1 107 47 8 171 10 141 12 150 16 123 187 133 31 61 77 191 102 202 201 239 182 248 209 113 196 207 93 146 221 64 158 49 83 57 139 217 218 174 100 87 216 22 45 11 238 50 59 105 205 28 90 4 174 64 115 221 29 143 88 239 25 55 219 221 123 247 65 190 242 107 90 69 203 130 187 191 114 173 221 107 66 161 105 136 54 232 253 159 174 111 240 249 188 69 16 29 242 187 51 67 40 182 85 29 43 90 6 238 22 59 40 101 36 96 17 65 135 184 155 127 35 107 171 2 83 68 176 160 188 201 226 215 217 22 42 94 242 71 211 69 179 202 87 5 117 220 188 61 24 28 185 139 23 144 107 61 140 111 62 177 254 95 14 20 205 218 254 39 192 0 85 75 56 201 129 50 9 35 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray;
			yourself);
		addChild: ((PRPage named: 'title')
			contents: '!!*/*';
			yourself);
		addChild: ((PRComponent named: 'search')
			componentClass: PRAjaxSearchWidget;
			yourself);
		addChild: ((PRComponent named: 'children')
			componentClass: PRChildrenWidget;
			write: 1 using: PRChildrenWidget descriptionLevel;
			yourself);
		addChild: ((PRPage named: 'sidebar')
			addChild: ((PRComponent named: 'navigation')
				componentClass: PRChildrenWidget;
				write: 2 using: PRChildrenWidget descriptionLevel;
				write: true using: PRChildrenWidget descriptionExpand;
				yourself);
			addChild: ((PRComponent named: 'views')
				componentClass: PRViewsWidget;
				yourself);
			addChild: ((PRComponent named: 'commands')
				componentClass: PRCommandsWidget;
				yourself);
			contents: '+navigation+
+views+
+commands+';
			yourself);
		addChild: ((PRComponent named: 'contents')
			componentClass: PRContentsWidget;
			write: '%c' using: PRContentsWidget descriptionHeading;
			yourself);
		addChild: ((PRPage named: 'footer')
			contents: 'Powered by *Seaside>http://www.seaside.st/*, *Magritte>http://www.lukas-renggli.ch/smalltalk/magritte*
		and *Pier>http://www.piercms.com/*.';
			yourself);
		contents: '{{{<div class="container">
	<div class="header column span-24">
		<div class="title column span-20">}}}+title+{{{</div>
		<div class="search column span-4 last">}}}+search+{{{</div>
		<div class="children column span-24 last">}}}+children+{{{</div>
	</div>
	<div class="body column span-24">
		<div class="sidebar column span-4">}}}+sidebar+{{{</div>
		<div class="contents column span-20 last">}}}+contents+{{{</div>
	</div>
	<div class="footer column span-24">}}}+footer+{{{</div>
</div>}}}';
	yourself! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 5/26/2009 17:31'!
embeddedComponents
	"Answer a list of embedded component links."
	
	^ self outgoingReferences embeddedComponents! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 10/25/2007 23:28'!
environment
	"Answer the (inherited) environment structure of the receiver, if none is available a default one is created."

	^ self localEnvironment ifNil: [
		self isRoot
			ifTrue: [ self newEnvironment ]
			ifFalse: [ self parent environment ] ]! !

!PRStructure methodsFor: '*pier-seaside-testing' stamp: 'lr 5/27/2009 16:04'!
hasContentsWidget
	^ self embeddedComponents anySatisfy: [ :each | each target componentClass includesBehavior: PRContentsWidget ]! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 6/22/2007 18:43'!
hideFromMenus
	^ self propertyAt: #hideFromMenus ifAbsentPut: [ false ]! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 6/22/2007 18:43'!
hideFromMenus: aBoolean
	^ self propertyAt: #hideFromMenus put: aBoolean! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'DamienPollet 11/13/2008 00:17'!
icon
	"Answer the (inherited) icon structure of the receiver,"

	| icon |
	icon := self localIcon.
	^ (icon notNil and: [icon file notNil])
		ifTrue: [ icon ]
		ifFalse: [
			self isRoot
				ifTrue: [ nil ]
				ifFalse: [ self parent icon ] ]! !

!PRStructure methodsFor: '*pier-seaside-testing' stamp: 'lr 5/12/2007 16:26'!
isComponent
	^ false! !

!PRStructure methodsFor: '*pier-seaside-testing' stamp: 'lr 3/9/2006 11:30'!
isValidView: aComponentClass in: aContext
	"Answer true if aComponentClass is a valid command in aContext."

	^ true! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 10/19/2007 14:33'!
localEnvironment
	^ self propertyAt: #localEnvironment ifAbsent: [ nil ]! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 10/25/2007 23:27'!
localEnvironment: aStructure
	self propertyAt: #localEnvironment put: aStructure! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'DamienPollet 11/13/2008 00:04'!
localIcon
	^ self propertyAt: #localIcon ifAbsent: [ nil ]! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'DamienPollet 11/12/2008 23:59'!
localIcon: aStructure
	self propertyAt: #localIcon put: aStructure! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 11/2/2007 17:18'!
localStyleSheet
	^ self propertyAt: #localStyleSheet ifAbsent: [ nil ]! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 11/2/2007 17:22'!
localStyleSheet: aStructure
	self propertyAt: #localStyleSheet put: aStructure! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 6/22/2007 18:36'!
menu
	^ self propertyAt: #menu ifAbsent: [ self title ]! !

!PRStructure methodsFor: '*pier-seaside-accessing' stamp: 'lr 6/22/2007 18:37'!
menu: aString
	(aString isNil or: [ aString isEmpty or: [ aString = self title ] ])
		ifTrue: [ self properties removeKey: #menu ifAbsent: [ ] ]
		ifFalse: [ self propertyAt: #menu put: aString ]! !

!PRStructure methodsFor: '*pier-seaside-configuration' stamp: 'lr 4/15/2009 17:46'!
newEnvironment
	self localEnvironment: self defaultEnvironment.
	self localStyleSheet: (self localEnvironment 
		childrenDecoration at: 'style.css').
	self childrenDecoration copy children do: [ :each |
		self localEnvironment name = each name
			ifTrue: [ self childrenDecoration remove: each ] ].
	self childrenDecoration add: self localEnvironment.
	^ self localEnvironment! !

!PRStructure methodsFor: '*pier-seaside-description' stamp: 'lr 5/22/2007 23:44'!
settingsDescription
	"Answer a description container of settings for the Seaside view."

	^ self description select: [ :each | each isSetting ]! !

!PRStructure methodsFor: '*pier-seaside-environment' stamp: 'lr 11/10/2008 22:24'!
styleSheets
	"Answer the a collection of stylesheets of the receiver."
	
	^ (self parents collect: [ :each | each localStyleSheet ])
		reject: [ :each | each isNil or: [ each file isNil ] ]! !

!PRStructure methodsFor: '*pier-seaside-configuration' stamp: 'lr 3/9/2006 11:30'!
viewComponentClass
	"Return the default view component class of the reciever."

	^ PRDefaultView! !

!PRLocationCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 8/11/2007 21:57'!
saveButton
	^ super saveButton value: self class label! !

!WARenderCanvas methodsFor: '*pier-seaside' stamp: 'lr 4/8/2009 20:16'!
rendererInstance
	"Answer an instance of the current render visitor. This instance is only valid during the current rendering pass."
	
	^ context properties
		at: #rendererInstance
		ifAbsentPut: [
			PRCurrentContext value 
				command rendererInstance ]! !

WADynamicVariable subclass: #PRContentsRendered
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Widgets'!

!PRContentsRendered class methodsFor: 'defaults' stamp: 'lr 10/28/2008 12:20'!
defaultValue
	^ false! !

WARequestFilter subclass: #PRContextFilter
	instanceVariableNames: 'presenter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Core'!

!PRContextFilter class methodsFor: 'instance creation' stamp: 'lr 1/11/2010 23:43'!
on: aPierFrame
	^ self basicNew initializeOn: aPierFrame! !

!PRContextFilter methodsFor: 'processing' stamp: 'lr 1/11/2010 23:43'!
handleFiltered: aRequestContext
	^ PRCurrentContext use: presenter during: [ super handleFiltered: aRequestContext ]! !

!PRContextFilter methodsFor: 'initialization' stamp: 'lr 1/11/2010 23:44'!
initializeOn: aPierFrame
	self initialize.
	presenter := aPierFrame! !

!WAAnchorTag methodsFor: '*pier-seaside-linking' stamp: 'NickAger 1/7/2010 18:57'!
goto: aContext
	"Initialize the receiver to properly activate aContext when being clicked. This includes a callback and a human-readable and bookmarkable url."

	aContext isValid
		ifFalse: [ self class: 'protected' ].
	(self attributes includesKey: 'title') 
		ifFalse: [ self title: aContext command label ].
	(aContext command isView not and: [ aContext command class accessKey notNil ])
		ifTrue: [ self attributes at: 'title' append: '(' , aContext command class accessKey greaseString , ')' ].
	self class: aContext structure cssClass.
	self url: (aContext urlOn: canvas)! !

!PRStructureDescription class methodsFor: '*pier-seaside-defaults' stamp: 'lr 3/9/2006 11:30'!
defaultComponentClasses
	^ Array with: PRStructureComponent! !

!PRViewCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 3/9/2006 11:30'!
asComponent
	^ self viewComponent! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'cdlm 8/7/2007 01:16'!
defaultViewComponentClass
	^ self context structure viewComponentClass! !

!PRViewCommand methodsFor: '*pier-seaside-protected' stamp: 'NickAger 1/8/2010 17:32'!
initialRequest: aRequest
	| viewClass |
	super initialRequest: aRequest.
	viewClass := PRPierFrame
		classFromRequest: aRequest
		name: 'view'
		base: PRViewComponent.
	(viewClass notNil and: [ viewClass isValidIn: self context ])
		ifTrue: [ self viewComponentClass: viewClass ].
	self viewComponent visiblePresentersDo: [ :each | each initialRequest: aRequest ].
	(self viewComponent isFullResponse)
		ifTrue: [ self viewComponent handle: WACurrentRequestContext value ]
! !

!PRViewCommand methodsFor: '*pier-seaside-testing' stamp: 'lr 7/13/2009 20:55'!
isRestful
	^ self viewComponent isRestful! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 1/29/2008 09:05'!
label
	^ self viewComponent label! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 4/8/2009 20:15'!
rendererInstance
	^ self viewComponent rendererInstance! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 2/20/2007 15:07'!
robots
	^ self viewComponent robots! !

!PRViewCommand methodsFor: '*pier-seaside-protected' stamp: 'lr 5/2/2009 11:28'!
sanitizeSeasideCache
	<sanitize>
	
	self properties removeKey: #viewComponent ifAbsent: [ ]! !

!PRViewCommand methodsFor: '*pier-seaside-protected' stamp: 'lr 2/23/2010 21:14'!
updateUrl: anUrl 
	super updateUrl: anUrl.
	self viewComponentClass = self defaultViewComponentClass 
		ifFalse: [ anUrl queryFields at: 'view' put: self viewComponentClass name greaseString ].
	self properties 
		at: #viewComponent
		ifPresent: [ :component | component updateUrl: anUrl ]! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
viewComponent
	^ self propertyAt: #viewComponent ifAbsentPut: [ self viewComponentClass on: self context ]! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
viewComponent: aViewComponent
	self propertyAt: #viewComponent put: aViewComponent! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 1/19/2008 20:36'!
viewComponentClass
	^ self propertyAt: #viewComponentClass ifAbsentPut: [ self defaultViewComponentClass ]! !

!PRViewCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 3/9/2006 11:30'!
viewComponentClass: aClass
	self propertyAt: #viewComponentClass put: aClass! !

!MADescription methodsFor: '*pier-seaside' stamp: 'lr 5/22/2007 23:43'!
beSetting
	self propertyAt: #setting put: true! !

!MADescription methodsFor: '*pier-seaside' stamp: 'lr 5/22/2007 23:44'!
isSetting
	^ self propertyAt: #setting ifAbsent: [ false ]! !

!PRChangeCommand methodsFor: '*pier-seaside' stamp: 'lr 11/6/2007 13:33'!
applyButton
	^ #apply -> 'Apply'! !

!PRChangeCommand methodsFor: '*pier-seaside' stamp: 'lr 11/6/2007 13:33'!
buttonSpecification
	^ Array with: self saveButton with: self applyButton with: self cancelButton! !

!PRChangeCommand methodsFor: '*pier-seaside' stamp: 'lr 4/9/2009 08:35'!
doAnswer
	super doAnswer.
	self structure isComponent
		ifTrue: [ self structure flush ]! !

PRChangeCommand subclass: #PRSettingsCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Model'!

!PRSettingsCommand commentStamp: 'lr 11/5/2007 21:40' prior: 0!
Command to edit settings of a structure. This command collects all the descriptions tagged as ==#beSetting==.!

!PRSettingsCommand class methodsFor: 'accessing' stamp: 'lr 10/19/2007 11:41'!
accessKey
	^ $t! !

!PRSettingsCommand class methodsFor: 'testing' stamp: 'lr 11/5/2007 21:44'!
isAbstract
	^ false! !

!PRSettingsCommand class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
label
	^ 'Settings'! !

!PRSettingsCommand methodsFor: 'accessing' stamp: 'lr 5/22/2007 23:51'!
description
	^ self structure settingsDescription! !

WAFileLibrary subclass: #PRBlueprintDebugLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Libraries'!

!PRBlueprintDebugLibrary methodsFor: 'uploaded' stamp: 'lr 1/29/2008 09:19'!
gridCss
	^ '.container { background: url(grid.png); }'! !

!PRBlueprintDebugLibrary methodsFor: 'uploaded' stamp: 'NickAger 1/12/2010 07:34'!
gridPng
	^ #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 40 0 0 0 18 8 6 0 0 0 41 31 156 166 0 0 0 4 115 66 73 84 8 8 8 8 124 8 100 136 0 0 0 9 112 72 89 115 0 0 10 240 0 0 10 240 1 66 172 52 152 0 0 0 28 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 70 105 114 101 119 111 114 107 115 32 67 83 51 152 214 70 3 0 0 0 72 73 68 65 84 72 137 99 124 241 254 247 127 134 1 0 226 2 44 68 169 99 162 177 59 40 6 163 14 164 20 140 58 144 82 48 234 64 74 193 168 3 41 5 163 14 164 20 140 58 144 82 48 234 64 74 193 160 119 32 11 227 175 119 3 98 241 171 87 196 169 3 0 78 179 7 223 243 172 62 238 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray! !

!PRBlueprintDebugLibrary methodsFor: 'accessing' stamp: 'tg 1/20/2008 12:39'!
selectorsToInclude
	^ #(gridCss)! !

WAFileLibrary subclass: #PRBlueprintLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Libraries'!

!PRBlueprintLibrary commentStamp: 'lr 1/11/2008 08:05' prior: 0!
!!Blueprint CSS Framework

Welcome to *Blueprint>http://code.google.com/p/blueprintcss*!! This is a CSS framework designed to cut down on your CSS development time. It gives you a solid foundation to build your own CSS on. Here are some of the features BP provides out-of-the-box:

- An easily customizable grid
- Sensible default typography
- A typographic baseline
- Perfected browser CSS reset
- A stylesheet for printing
- Absolutely no bloat!!

!!!!License
Copyright (c) 2007 Olav Bjorkoy (http://bjorkoy.com)

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sub-license, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice, and every other copyright notice found in this software, and all the attributions in every file, and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.!

!PRBlueprintLibrary methodsFor: 'uploaded' stamp: 'lr 3/11/2008 10:05'!
ieCss
	^ '/* -----------------------------------------------------------------------

   Blueprint CSS Framework 0.7.1
   http://blueprintcss.googlecode.com

   * Copyright (c) 2007-2008. See LICENSE for more info.
   * See README for instructions on how to use Blueprint.
   * For credits and origins, see AUTHORS.
   * This is a compressed file. See the sources in the ''src'' directory.

----------------------------------------------------------------------- */

/* ie.css */
body {text-align:center;}
.container {text-align:left;}
* html .column {overflow-x:hidden;}
* html legend {margin:-18px -8px 16px 0;padding:0;}
ol {margin-left:2em;}
sup {vertical-align:text-top;}
sub {vertical-align:text-bottom;}
html>body p code {*white-space:normal;}
hr {margin:-8px auto 11px;}'! !

!PRBlueprintLibrary methodsFor: 'uploaded' stamp: 'lr 3/11/2008 10:04'!
printCss
	^ '/* -----------------------------------------------------------------------

   Blueprint CSS Framework 0.7.1
   http://blueprintcss.googlecode.com

   * Copyright (c) 2007-2008. See LICENSE for more info.
   * See README for instructions on how to use Blueprint.
   * For credits and origins, see AUTHORS.
   * This is a compressed file. See the sources in the ''src'' directory.

----------------------------------------------------------------------- */

/* print.css */
body {line-height:1.5;font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;color:#000;background:none;font-size:10pt;}
.container {background:none;}
hr {background:#ccc;color:#ccc;width:100%;height:2px;margin:2em 0;padding:0;border:none;}
hr.space {background:#fff;color:#fff;}
h1, h2, h3, h4, h5, h6 {font-family:"Helvetica Neue", Arial, "Lucida Grande", sans-serif;}
code {font:.9em "Courier New", Monaco, Courier, monospace;}
img {float:left;margin:1.5em 1.5em 1.5em 0;}
a img {border:none;}
p img.top {margin-top:0;}
blockquote {margin:1.5em;padding:1em;font-style:italic;font-size:.9em;}
.small {font-size:.9em;}
.large {font-size:1.1em;}
.quiet {color:#999;}
.hide {display:none;}
a:link, a:visited {background:transparent;font-weight:700;text-decoration:underline;}
a:link:after, a:visited:after {content:" (" attr(href) ") ";font-size:90%;}'! !

!PRBlueprintLibrary methodsFor: 'uploaded' stamp: 'lr 5/28/2008 12:28'!
screenCss
	^ '/* -----------------------------------------------------------------------

   Blueprint CSS Framework 0.7.1
   http://blueprintcss.googlecode.com

   * Copyright (c) 2007-2008. See LICENSE for more info.
   * See README for instructions on how to use Blueprint.
   * For credits and origins, see AUTHORS.
   * This is a compressed file. See the sources in the ''src'' directory.

----------------------------------------------------------------------- */

/* reset.css */
html, body, div, span, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, code, del, dfn, em, img, q, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td {margin:0;padding:0;border:0;font-weight:inherit;font-style:inherit;font-size:100%;font-family:inherit;vertical-align:baseline;}
body {line-height:1.5;}
table {border-collapse:separate;border-spacing:0;}
caption, th, td {text-align:left;font-weight:normal;}
table, td, th {vertical-align:middle;}
blockquote:before, blockquote:after, q:before, q:after {content:"";}
blockquote, q {quotes:"" "";}
a img {border:none;}

/* typography.css */
body {font-size:75%;color:#222;background:#fff;font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;}
h1, h2, h3, h4, h5, h6 {font-weight:normal;color:#111;}
h1 {font-size:3em;line-height:1;margin-bottom:0.5em;}
h2 {font-size:2em;margin-bottom:0.75em;}
h3 {font-size:1.5em;line-height:1;margin-bottom:1em;}
h4 {font-size:1.2em;line-height:1.25;margin-bottom:1.25em;height:1.25em;}
h5 {font-size:1em;font-weight:bold;margin-bottom:1.5em;}
h6 {font-size:1em;font-weight:bold;}
h1 img, h2 img, h3 img, h4 img, h5 img, h6 img {margin:0;}
p {margin:0 0 1.5em;}
p img {float:left;margin:1.5em 1.5em 1.5em 0;padding:0;}
p img.right {float:right;margin:1.5em 0 1.5em 1.5em;}
a:focus, a:hover {color:#000;}
a {color:#009;text-decoration:underline;}
blockquote {margin:1.5em;color:#666;font-style:italic;}
strong {font-weight:bold;}
em, dfn {font-style:italic;}
dfn {font-weight:bold;}
sup, sub {line-height:0;}
abbr, acronym {border-bottom:1px dotted #666;}
address {margin:0 0 1.5em;font-style:italic;}
del {color:#666;}
pre, code {margin:1.5em 0;white-space:pre;}
pre, code, tt {font:1em ''andale mono'', ''lucida console'', monospace;line-height:1.5;}
li ul, li ol {margin:0 1.5em;}
ul, ol {margin:0 1.5em 1.5em 1.5em;}
ul {list-style-type:disc;}
ol {list-style-type:decimal;}
dl {margin:0 0 1.5em 0;}
dl dt {font-weight:bold;}
dd {margin-left:1.5em;}
table {margin-bottom:1.4em;width:100%;}
th {font-weight:bold;background:#C3D9FF;}
th, td {padding:4px 10px 4px 5px;}
tr.even td {background:#E5ECF9;}
tfoot {font-style:italic;}
caption {background:#eee;}
.small {font-size:.8em;margin-bottom:1.875em;line-height:1.875em;}
.large {font-size:1.2em;line-height:2.5em;margin-bottom:1.25em;}
.hide {display:none;}
.quiet {color:#666;}
.loud {color:#000;}
.highlight {background:#ff0;}
.added {background:#060;color:#fff;}
.removed {background:#900;color:#fff;}
.first {margin-left:0;padding-left:0;}
.last {margin-right:0;padding-right:0;}
.top {margin-top:0;padding-top:0;}
.bottom {margin-bottom:0;padding-bottom:0;}

/* grid.css */
.container {width:950px;margin:0 auto;}
.showgrid {background:url(src/grid.png);}
body {margin:1.5em 0;}
div.span-1, div.span-2, div.span-3, div.span-4, div.span-5, div.span-6, div.span-7, div.span-8, div.span-9, div.span-10, div.span-11, div.span-12, div.span-13, div.span-14, div.span-15, div.span-16, div.span-17, div.span-18, div.span-19, div.span-20, div.span-21, div.span-22, div.span-23, div.span-24 {float:left;margin-right:10px;}
div.last {margin-right:0;}
.span-1 {width:30px;}
.span-2 {width:70px;}
.span-3 {width:110px;}
.span-4 {width:150px;}
.span-5 {width:190px;}
.span-6 {width:230px;}
.span-7 {width:270px;}
.span-8 {width:310px;}
.span-9 {width:350px;}
.span-10 {width:390px;}
.span-11 {width:430px;}
.span-12 {width:470px;}
.span-13 {width:510px;}
.span-14 {width:550px;}
.span-15 {width:590px;}
.span-16 {width:630px;}
.span-17 {width:670px;}
.span-18 {width:710px;}
.span-19 {width:750px;}
.span-20 {width:790px;}
.span-21 {width:830px;}
.span-22 {width:870px;}
.span-23 {width:910px;}
.span-24, div.span-24 {width:950px;margin:0;}
.append-1 {padding-right:40px;}
.append-2 {padding-right:80px;}
.append-3 {padding-right:120px;}
.append-4 {padding-right:160px;}
.append-5 {padding-right:200px;}
.append-6 {padding-right:240px;}
.append-7 {padding-right:280px;}
.append-8 {padding-right:320px;}
.append-9 {padding-right:360px;}
.append-10 {padding-right:400px;}
.append-11 {padding-right:440px;}
.append-12 {padding-right:480px;}
.append-13 {padding-right:520px;}
.append-14 {padding-right:560px;}
.append-15 {padding-right:600px;}
.append-16 {padding-right:640px;}
.append-17 {padding-right:680px;}
.append-18 {padding-right:720px;}
.append-19 {padding-right:760px;}
.append-20 {padding-right:800px;}
.append-21 {padding-right:840px;}
.append-22 {padding-right:880px;}
.append-23 {padding-right:920px;}
.prepend-1 {padding-left:40px;}
.prepend-2 {padding-left:80px;}
.prepend-3 {padding-left:120px;}
.prepend-4 {padding-left:160px;}
.prepend-5 {padding-left:200px;}
.prepend-6 {padding-left:240px;}
.prepend-7 {padding-left:280px;}
.prepend-8 {padding-left:320px;}
.prepend-9 {padding-left:360px;}
.prepend-10 {padding-left:400px;}
.prepend-11 {padding-left:440px;}
.prepend-12 {padding-left:480px;}
.prepend-13 {padding-left:520px;}
.prepend-14 {padding-left:560px;}
.prepend-15 {padding-left:600px;}
.prepend-16 {padding-left:640px;}
.prepend-17 {padding-left:680px;}
.prepend-18 {padding-left:720px;}
.prepend-19 {padding-left:760px;}
.prepend-20 {padding-left:800px;}
.prepend-21 {padding-left:840px;}
.prepend-22 {padding-left:880px;}
.prepend-23 {padding-left:920px;}
div.border {padding-right:4px;margin-right:5px;border-right:1px solid #eee;}
div.colborder {padding-right:24px;margin-right:25px;border-right:1px solid #eee;}
.pull-1 {margin-left:-40px;}
.pull-2 {margin-left:-80px;}
.pull-3 {margin-left:-120px;}
.pull-4 {margin-left:-160px;}
.pull-5 {margin-left:-200px;}
.pull-6 {margin-left:-240px;}
.pull-7 {margin-left:-280px;}
.pull-8 {margin-left:-320px;}
.pull-9 {margin-left:-360px;}
.pull-10 {margin-left:-400px;}
.pull-11 {margin-left:-440px;}
.pull-12 {margin-left:-480px;}
.pull-13 {margin-left:-520px;}
.pull-14 {margin-left:-560px;}
.pull-15 {margin-left:-600px;}
.pull-16 {margin-left:-640px;}
.pull-17 {margin-left:-680px;}
.pull-18 {margin-left:-720px;}
.pull-19 {margin-left:-760px;}
.pull-20 {margin-left:-800px;}
.pull-21 {margin-left:-840px;}
.pull-22 {margin-left:-880px;}
.pull-23 {margin-left:-920px;}
.pull-24 {margin-left:-960px;}
.pull-1, .pull-2, .pull-3, .pull-4, .pull-5, .pull-6, .pull-7, .pull-8, .pull-9, .pull-10, .pull-11, .pull-12, .pull-13, .pull-14, .pull-15, .pull-16, .pull-17, .pull-18, .pull-19, .pull-20, .pull-21, .pull-22, .pull-23, .pull-24 {float:left;position:relative;}
.push-1 {margin:0 -40px 1.5em 40px;}
.push-2 {margin:0 -80px 1.5em 80px;}
.push-3 {margin:0 -120px 1.5em 120px;}
.push-4 {margin:0 -160px 1.5em 160px;}
.push-5 {margin:0 -200px 1.5em 200px;}
.push-6 {margin:0 -240px 1.5em 240px;}
.push-7 {margin:0 -280px 1.5em 280px;}
.push-8 {margin:0 -320px 1.5em 320px;}
.push-9 {margin:0 -360px 1.5em 360px;}
.push-10 {margin:0 -400px 1.5em 400px;}
.push-11 {margin:0 -440px 1.5em 440px;}
.push-12 {margin:0 -480px 1.5em 480px;}
.push-13 {margin:0 -520px 1.5em 520px;}
.push-14 {margin:0 -560px 1.5em 560px;}
.push-15 {margin:0 -600px 1.5em 600px;}
.push-16 {margin:0 -640px 1.5em 640px;}
.push-17 {margin:0 -680px 1.5em 680px;}
.push-18 {margin:0 -720px 1.5em 720px;}
.push-19 {margin:0 -760px 1.5em 760px;}
.push-20 {margin:0 -800px 1.5em 800px;}
.push-21 {margin:0 -840px 1.5em 840px;}
.push-22 {margin:0 -880px 1.5em 880px;}
.push-23 {margin:0 -920px 1.5em 920px;}
.push-24 {margin:0 -960px 1.5em 960px;}
.push-1, .push-2, .push-3, .push-4, .push-5, .push-6, .push-7, .push-8, .push-9, .push-10, .push-11, .push-12, .push-13, .push-14, .push-15, .push-16, .push-17, .push-18, .push-19, .push-20, .push-21, .push-22, .push-23, .push-24 {float:right;position:relative;}
.box {padding:1.5em;margin-bottom:1.5em;background:#E5ECF9;}
hr {background:#ddd;color:#ddd;clear:both;float:none;width:100%;height:.1em;margin:0 0 1.45em;border:none;}
hr.space {background:#fff;color:#fff;}
.clearfix:after, .container:after {content:".";display:block;height:0;clear:both;visibility:hidden;}
.clearfix, .container {display:inline-block;}
* html .clearfix, * html .container {height:1%;}
.clearfix, .container {display:block;}
.clear {clear:both;}

/* forms.css */
label {font-weight:bold;}
fieldset {padding:1.4em;margin:0 0 1.5em 0;border:1px solid #ccc;}
legend {font-weight:bold;font-size:1.2em;}
input.text, input.title, input.password, textarea, select {margin:0.5em 0;border:1px solid #bbb;}
input.text:focus, input.title:focus, input.password:focus, textarea:focus, select:focus {border:1px solid #666;}
input.text, input.password, input.title {width:300px;padding:5px;}
input.title {font-size:1.5em;}
textarea {width:390px;height:250px;padding:5px;}
.error, .notice, .success {padding:.8em;margin-bottom:1em;border:2px solid #ddd;}
.error {background:#FBE3E4;color:#8a1f11;border-color:#FBC2C4;}
.notice {background:#FFF6BF;color:#514721;border-color:#FFD324;}
.success {background:#E6EFC2;color:#264409;border-color:#C6D880;}
.error a {color:#8a1f11;}
.notice a {color:#514721;}
.success a {color:#264409;}'! !

!PRBlueprintLibrary methodsFor: 'processing' stamp: 'lr 5/28/2008 12:26'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot stylesheet
		addScreen;
		addProjection;
		url: (self urlOf: #screenCss).
	aHtmlRoot stylesheet
		addPrint;
		url: (self urlOf: #printCss).
	aHtmlRoot if ie do: [
		aHtmlRoot stylesheet
			addScreen;
			addProjection;
			url: (self urlOf: #ieCss) ]! !

WAFileLibrary subclass: #PRJavaScriptSupport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Libraries'!

!PRJavaScriptSupport methodsFor: 'files' stamp: 'lr 3/17/2009 23:17'!
pierJs
	^ '(function(){
	var load = function(event) {
		if (document.forms.length > 0) {
			var elements = document.forms[0].elements;
			for (var i = 0; i < elements.length; i++) {
				if (elements[i].parentNode.tagName == "TD") {
					elements[i].focus();
					break;
				}
			}
		}
	};
	if (window.addEventListener)
		window.addEventListener("load", load, false)
	else
		window.attachEvent("onload", load);
})();'! !

!PRJavaScriptSupport methodsFor: 'accessing' stamp: 'lr 10/1/2008 22:59'!
selectorsToInclude
	^ #(pierJs)! !

WAFileLibrary subclass: #PRNarrowBlueprintDebugLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Libraries'!

!PRNarrowBlueprintDebugLibrary methodsFor: 'uploaded' stamp: 'tg 8/27/2008 08:49'!
gridCss
	^ '.container { background: url(grid.png); }'! !

!PRNarrowBlueprintDebugLibrary methodsFor: 'uploaded' stamp: 'NickAger 1/12/2010 07:35'!
gridPng
	^ #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 30 0 0 0 18 2 3 0 0 0 49 156 134 74 0 0 0 9 80 76 84 69 255 255 255 243 245 247 230 230 236 122 197 101 242 0 0 0 20 73 68 65 84 8 153 99 8 5 1 6 32 24 108 140 85 16 176 0 0 176 102 33 128 242 86 90 33 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray! !

!PRNarrowBlueprintDebugLibrary methodsFor: 'accessing' stamp: 'tg 8/27/2008 08:50'!
selectorsToInclude
	^ #(gridCss)! !

WAFileLibrary subclass: #PRNarrowBlueprintLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Seaside-Libraries'!

!PRNarrowBlueprintLibrary methodsFor: 'uploaded' stamp: 'tg 8/27/2008 00:51'!
ieCss
	^ '/* -----------------------------------------------------------------------

   Blueprint CSS Framework 0.7.1
   http://blueprintcss.googlecode.com

   * Copyright (c) 2007-2008. See LICENSE for more info.
   * See README for instructions on how to use Blueprint.
   * For credits and origins, see AUTHORS.
   * This is a compressed file. See the sources in the ''src'' directory.

----------------------------------------------------------------------- */

/* ie.css */
body {text-align:center;}
.container {text-align:left;}
* html .column {overflow-x:hidden;}
* html legend {margin:-18px -8px 16px 0;padding:0;}
ol {margin-left:2em;}
sup {vertical-align:text-top;}
sub {vertical-align:text-bottom;}
html>body p code {*white-space:normal;}
hr {margin:-8px auto 11px;}'! !

!PRNarrowBlueprintLibrary methodsFor: 'uploaded' stamp: 'tg 9/6/2008 20:46'!
patchCss

	^
'input.text, input.title, input.password, textarea, select {margin:0.5em 0;border:1px solid #bbb;}
input.text:focus, input.title:focus, input.password:focus, textarea:focus, select:focus {border:1px solid #666;}
input.text, input.password, input.title {width:300px;padding:5px;}'! !

!PRNarrowBlueprintLibrary methodsFor: 'uploaded' stamp: 'tg 8/27/2008 00:50'!
printCss
	^ '/* -----------------------------------------------------------------------

   Blueprint CSS Framework 0.7.1
   http://blueprintcss.googlecode.com

   * Copyright (c) 2007-2008. See LICENSE for more info.
   * See README for instructions on how to use Blueprint.
   * For credits and origins, see AUTHORS.
   * This is a compressed file. See the sources in the ''src'' directory.

----------------------------------------------------------------------- */

/* print.css */
body {line-height:1.5;font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;color:#000;background:none;font-size:10pt;}
.container {background:none;}
hr {background:#ccc;color:#ccc;width:100%;height:2px;margin:2em 0;padding:0;border:none;}
hr.space {background:#fff;color:#fff;}
h1, h2, h3, h4, h5, h6 {font-family:"Helvetica Neue", Arial, "Lucida Grande", sans-serif;}
code {font:.9em "Courier New", Monaco, Courier, monospace;}
img {float:left;margin:1.5em 1.5em 1.5em 0;}
a img {border:none;}
p img.top {margin-top:0;}
blockquote {margin:1.5em;padding:1em;font-style:italic;font-size:.9em;}
.small {font-size:.9em;}
.large {font-size:1.1em;}
.quiet {color:#999;}
.hide {display:none;}
a:link, a:visited {background:transparent;font-weight:700;text-decoration:underline;}
a:link:after, a:visited:after {content:" (" attr(href) ") ";font-size:90%;}'! !

!PRNarrowBlueprintLibrary methodsFor: 'uploaded' stamp: 'tg 9/6/2008 20:36'!
screenCss
	^ 
'/* 
  BLUEPRINT CSS
   * Filename:        compressed.css
   * Version:         0.7.1 (2008-02-25) YYYY-MM-DD
   * Website:         http://code.google.com/p/blueprintcss/
   
   Generated by:
    * Blueprint CSS Grid Generator (2008-09-06) [http://kematzy.com/blueprint-generator/]
    
  == STRUCTURE: ========================
   * Page width:            710 px
   * Number of columns:     24
   * Column width:          20 px
   * Margin width:          10 px
  ======================================
   
*/

/* reset.css */
html, body, div, span, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, code, del, dfn, em, img, q, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td {margin:0;padding:0;border:0;font-weight:inherit;font-style:inherit;font-size:100%;font-family:inherit;vertical-align:baseline;}
body {line-height:1.5;}
table {border-collapse:separate;border-spacing:0;}
caption, th, td {text-align:left;font-weight:normal;}
table, td, th {vertical-align:middle;}
blockquote:before, blockquote:after, q:before, q:after {content:"";}
blockquote, q {quotes:"" "";}
a img {border:none;}

/* typography.css */
body {font-size:75%;color:#222;background:#fff;font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;}
h1, h2, h3, h4, h5, h6 {font-weight:normal;color:#111;}
h1 {font-size:3em;line-height:1;margin-bottom:0.5em;}
h2 {font-size:2em;margin-bottom:0.75em;}
h3 {font-size:1.5em;line-height:1;margin-bottom:1em;}
h4 {font-size:1.2em;line-height:1.25;margin-bottom:1.25em;height:1.25em;}
h5 {font-size:1em;font-weight:bold;margin-bottom:1.5em;}
h6 {font-size:1em;font-weight:bold;}
h1 img, h2 img, h3 img, h4 img, h5 img, h6 img {margin:0;}
p {margin:0 0 1.5em;}
p img {float:left;margin:1.5em 1.5em 1.5em 0;padding:0;}
p img.right {float:right;margin:1.5em 0 1.5em 1.5em;}
a:focus, a:hover {color:#000;}
a {color:#009;text-decoration:underline;}
blockquote {margin:1.5em;color:#666;font-style:italic;}
strong {font-weight:bold;}
em, dfn {font-style:italic;}
dfn {font-weight:bold;}
sup, sub {line-height:0;}
abbr, acronym {border-bottom:1px dotted #666;}
address {margin:0 0 1.5em;font-style:italic;}
del {color:#666;}
pre, code {margin:1.5em 0;white-space:pre;}
pre, code, tt {font:1em ''andale mono'', ''lucida console'', monospace;line-height:1.5;}
li ul, li ol {margin:0 1.5em;}
ul, ol {margin:0 1.5em 1.5em 1.5em;}
ul {list-style-type:disc;}
ol {list-style-type:decimal;}
dl {margin:0 0 1.5em 0;}
dl dt {font-weight:bold;}
dd {margin-left:1.5em;}
table {margin-bottom:1.4em;width:100%;}
th {font-weight:bold;background:#C3D9FF;}
th, td {padding:4px 10px 4px 5px;}
tr.even td {background:#E5ECF9;}
tfoot {font-style:italic;}
caption {background:#eee;}
.small {font-size:.8em;margin-bottom:1.875em;line-height:1.875em;}
.large {font-size:1.2em;line-height:2.5em;margin-bottom:1.25em;}
.hide {display:none;}
.quiet {color:#666;}
.loud {color:#000;}
.highlight {background:#ff0;}
.added {background:#060;color:#fff;}
.removed {background:#900;color:#fff;}
.first {margin-left:0;padding-left:0;}
.last {margin-right:0;padding-right:0;}
.top {margin-top:0;padding-top:0;}
.bottom {margin-bottom:0;padding-bottom:0;}

/* grid.css */
.container {width:710px;margin:0 auto;}
.showgrid {background:url(src/grid.png);}
body {margin:1.5em 0;}
div.span-1, div.span-2, div.span-3, div.span-4, div.span-5, div.span-6, div.span-7, div.span-8, div.span-9, div.span-10, div.span-11, div.span-12, div.span-13, div.span-14, div.span-15, div.span-16, div.span-17, div.span-18, div.span-19, div.span-20, div.span-21, div.span-22, div.span-23, div.span-24 {float:left;margin-right: 10px;}

div.last {margin-right:0;}
.span-1  { width: 20px;}
.span-2  { width: 50px;}
.span-3  { width: 80px;}
.span-4  { width: 110px;}
.span-5  { width: 140px;}
.span-6  { width: 170px;}
.span-7  { width: 200px;}
.span-8  { width: 230px;}
.span-9  { width: 260px;}
.span-10 { width: 290px;}
.span-11 { width: 320px;}
.span-12 { width: 350px;}
.span-13 { width: 380px;}
.span-14 { width: 410px;}
.span-15 { width: 440px;}
.span-16 { width: 470px;}
.span-17 { width: 500px;}
.span-18 { width: 530px;}
.span-19 { width: 560px;}
.span-20 { width: 590px;}
.span-21 { width: 620px;}
.span-22 { width: 650px;}
.span-23 { width: 680px;}
.span-24, div.span-24 { width: 710px; margin: 0; }

.append-1  { padding-right: 30px;}
.append-2  { padding-right: 60px;}
.append-3  { padding-right: 90px;}
.append-4  { padding-right: 120px;}
.append-5  { padding-right: 150px;}
.append-6  { padding-right: 180px;}
.append-7  { padding-right: 210px;}
.append-8  { padding-right: 240px;}
.append-9  { padding-right: 270px;}
.append-10 { padding-right: 300px;}
.append-11 { padding-right: 330px;}
.append-12 { padding-right: 360px;}
.append-13 { padding-right: 390px;}
.append-14 { padding-right: 420px;}
.append-15 { padding-right: 450px;}
.append-16 { padding-right: 480px;}
.append-17 { padding-right: 510px;}
.append-18 { padding-right: 540px;}
.append-19 { padding-right: 570px;}
.append-20 { padding-right: 600px;}
.append-21 { padding-right: 630px;}
.append-22 { padding-right: 660px;}
.append-23 { padding-right: 690px;}

.prepend-1  { padding-left: 30px;}
.prepend-2  { padding-left: 60px;}
.prepend-3  { padding-left: 90px;}
.prepend-4  { padding-left: 120px;}
.prepend-5  { padding-left: 150px;}
.prepend-6  { padding-left: 180px;}
.prepend-7  { padding-left: 210px;}
.prepend-8  { padding-left: 240px;}
.prepend-9  { padding-left: 270px;}
.prepend-10 { padding-left: 300px;}
.prepend-11 { padding-left: 330px;}
.prepend-12 { padding-left: 360px;}
.prepend-13 { padding-left: 390px;}
.prepend-14 { padding-left: 420px;}
.prepend-15 { padding-left: 450px;}
.prepend-16 { padding-left: 480px;}
.prepend-17 { padding-left: 510px;}
.prepend-18 { padding-left: 540px;}
.prepend-19 { padding-left: 570px;}
.prepend-20 { padding-left: 600px;}
.prepend-21 { padding-left: 630px;}
.prepend-22 { padding-left: 660px;}
.prepend-23 { padding-left: 690px;}

div.border{padding-right:4px;margin-right:5px;border-right:1px solid #eee;}
div.colborder { padding-right:16px;margin-right:15px;border-right:1px solid #eee;}
.pull-1 { margin-left: -30px;}
.pull-2 { margin-left: -60px;}
.pull-3 { margin-left: -90px;}
.pull-4 { margin-left: -120px;}
.pull-5 { margin-left: -150px;}
.pull-6 { margin-left: -180px;}
.pull-7 { margin-left: -210px;}
.pull-8 { margin-left: -240px;}
.pull-9 { margin-left: -270px;}
.pull-10 { margin-left: -300px;}
.pull-11 { margin-left: -330px;}
.pull-12 { margin-left: -360px;}
.pull-13 { margin-left: -390px;}
.pull-14 { margin-left: -420px;}
.pull-15 { margin-left: -450px;}
.pull-16 { margin-left: -480px;}
.pull-17 { margin-left: -510px;}
.pull-18 { margin-left: -540px;}
.pull-19 { margin-left: -570px;}
.pull-20 { margin-left: -600px;}
.pull-21 { margin-left: -630px;}
.pull-22 { margin-left: -660px;}
.pull-23 { margin-left: -690px;}
.pull-24 { margin-left: -720px;}

.pull-1, .pull-2, .pull-3, .pull-4, .pull-5, .pull-6, .pull-7, .pull-8, .pull-9, .pull-10, .pull-11, .pull-12, .pull-13, .pull-14, .pull-15, .pull-16, .pull-17, .pull-18, .pull-19, .pull-20, .pull-21, .pull-22, .pull-23, .pull-24 {float:left;position:relative;}

.push-1 { margin: 0 -30px 1.5em 30px;}
.push-2 { margin: 0 -60px 1.5em 60px;}
.push-3 { margin: 0 -90px 1.5em 90px;}
.push-4 { margin: 0 -120px 1.5em 120px;}
.push-5 { margin: 0 -150px 1.5em 150px;}
.push-6 { margin: 0 -180px 1.5em 180px;}
.push-7 { margin: 0 -210px 1.5em 210px;}
.push-8 { margin: 0 -240px 1.5em 240px;}
.push-9 { margin: 0 -270px 1.5em 270px;}
.push-10 { margin: 0 -300px 1.5em 300px;}
.push-11 { margin: 0 -330px 1.5em 330px;}
.push-12 { margin: 0 -360px 1.5em 360px;}
.push-13 { margin: 0 -390px 1.5em 390px;}
.push-14 { margin: 0 -420px 1.5em 420px;}
.push-15 { margin: 0 -450px 1.5em 450px;}
.push-16 { margin: 0 -480px 1.5em 480px;}
.push-17 { margin: 0 -510px 1.5em 510px;}
.push-18 { margin: 0 -540px 1.5em 540px;}
.push-19 { margin: 0 -570px 1.5em 570px;}
.push-20 { margin: 0 -600px 1.5em 600px;}
.push-21 { margin: 0 -630px 1.5em 630px;}
.push-22 { margin: 0 -660px 1.5em 660px;}
.push-23 { margin: 0 -690px 1.5em 690px;}
.push-24 { margin: 0 -720px 1.5em 720px;}

.push-1, .push-2, .push-3, .push-4, .push-5, .push-6, .push-7, .push-8, .push-9, .push-10, .push-11, .push-12, .push-13, .push-14, .push-15, .push-16, .push-17, .push-18, .push-19, .push-20, .push-21, .push-22, .push-23, .push-24 {float:right;position:relative;}

.box {padding:1.5em;margin-bottom:1.5em;background:#E5ECF9;}
hr {background:#ddd;color:#ddd;clear:both;float:none;width:100%;height:.1em;margin:0 0 1.45em;border:none;}
hr.space {background:#fff;color:#fff;}
.clearfix:after, .container:after {content:".";display:block;height:0;clear:both;visibility:hidden;}
.clearfix, .container {display:inline-block;}
* html .clearfix, * html .container {height:1%;}
.clearfix, .container {display:block;}
.clear {clear:both;}

/* forms.css */
label {font-weight:bold;}
fieldset {padding:1.4em;margin:0 0 1.5em 0;border:1px solid #ccc;}
legend {font-weight:bold;font-size:1.2em;}
input.text, input.title, textarea, select {margin:0.5em 0;border:1px solid #bbb;}
input.text:focus, input.title:focus, textarea:focus, select:focus {border:1px solid #666;}
input.text, input.title {width:300px;padding:5px;}
input.title {font-size:1.5em;}
textarea {width:390px;height:250px;padding:5px;}
.error, .notice, .success {padding:.8em;margin-bottom:1em;border:2px solid #ddd;}
.error {background:#FBE3E4;color:#8a1f11;border-color:#FBC2C4;}
.notice {background:#FFF6BF;color:#514721;border-color:#FFD324;}
.success {background:#E6EFC2;color:#264409;border-color:#C6D880;}
.error a {color:#8a1f11;}
.notice a {color:#514721;}
.success a {color:#264409;}'! !

!PRNarrowBlueprintLibrary methodsFor: 'processing' stamp: 'tg 9/6/2008 20:46'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot stylesheet
		addScreen;
		addProjection;
		url: (self urlOf: #screenCss).
	aHtmlRoot stylesheet
		addScreen;
		addProjection;
		url: (self urlOf: #patchCss).
	aHtmlRoot stylesheet
		addPrint;
		url: (self urlOf: #printCss).
	aHtmlRoot if ie do: [
		aHtmlRoot stylesheet
			addScreen;
			addProjection;
			url: (self urlOf: #ieCss) ]! !

!PRDocumentDescription class methodsFor: '*pier-seaside-defaults' stamp: 'lr 9/9/2009 19:11'!
defaultComponentClasses
	^ Array with: PRDocumentComponent! !

!PRDocumentDescription class methodsFor: '*pier-seaside-defaults' stamp: 'lr 1/10/2008 16:13'!
defaultCssClasses
	^ super defaultCssClasses add: 'wiki'; yourself! !

!PRCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 11/6/2007 13:29'!
asComponent
	"Answer a compoent to configure the edit operation of the receiver. This does not require to be a Magritte component, but in most cases it is."

	^ super asComponent
		addValidatedForm: self buttonSpecification;
		yourself! !

!PRCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 11/6/2007 13:30'!
buttonSpecification
	^ Array with: self saveButton with: self cancelButton! !

!PRCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 8/11/2007 21:55'!
cancelButton
	^ #cancel -> 'Cancel'! !

!PRCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 9/12/2007 18:18'!
cssClass
	"Answer a CSS class that should be used in the body-tag when dis command is rendered."

	^ self class fullName asLowercase allButFirst: 2! !

!PRCommand methodsFor: '*pier-seaside-protected' stamp: 'lr 8/3/2007 17:47'!
initialRequest: aRequest
	"Override this to extract additional data from the request. This is only called for requests that don't have a valid session yet.'"! !

!PRCommand methodsFor: '*pier-seaside-testing' stamp: 'lr 7/13/2009 20:54'!
isRestful
	^ false! !

!PRCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 4/8/2009 20:18'!
rendererInstance
	"Answer a renderer, that is the visitor that is used to display the page."

	^ PRViewRenderer new! !

!PRCommand methodsFor: '*pier-seaside-accessing' stamp: 'lr 2/20/2007 15:07'!
robots
	^ 'noindex,follow'! !

!PRCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 8/11/2007 21:55'!
saveButton
	^ #save -> 'Save'! !

!PRCommand methodsFor: '*pier-seaside-protected' stamp: 'lr 9/12/2007 18:18'!
updateRoot: aHtmlRoot
	"Override customize the <head> section of an HTML document."

	aHtmlRoot meta
		name: 'robots';
		content: self robots.
	aHtmlRoot bodyAttributes
		addClass: self cssClass! !

!PRCommand methodsFor: '*pier-seaside-protected' stamp: 'lr 2/23/2010 21:12'!
updateUrl: anUrl 
	"Override to customize the URL used to display this command."
	
	self class = self structure defaultCommandClass 
		ifFalse: [ anUrl queryFields at: 'command' put: self class name greaseString ]! !

!PRRemoveCommand methodsFor: '*pier-seaside-converting' stamp: 'lr 9/9/2009 19:30'!
saveButton
	^ super saveButton value: self class label! !

!GRCodecStream methodsFor: '*pier-seaside' stamp: 'jf 11/2/2009 20:39'!
isEmpty
	self flag: #todo. "Remove this method - used by PRDocumentWriter>>start:"
	^ stream isEmpty! !

!GRCodecStream methodsFor: '*pier-seaside' stamp: 'jf 11/2/2009 20:37'!
position
	self flag: #todo. "Remove this method - used by PRDocumentWriter>>start:"
	^ stream position! !

!GRCodecStream methodsFor: '*pier-seaside' stamp: 'jf 11/2/2009 20:37'!
position: anInteger
	self flag: #todo. "Remove this method - used by PRDocumentWriter>>start:"
	stream position: anInteger! !

!PRValueLink methodsFor: '*pier-seaside' stamp: 'jf 9/30/2009 01:10'!
redirectIn: aContext 
	" ignore the request if this is not the default view, e.g. an rss changes feed "
	| link url |
	<value: 'redirect' comment: 'Redirect to a different location, specified in the ''to'' parameter.'>
	(aContext command isView and: 
		[ aContext command viewComponentClass = aContext command defaultViewComponentClass ]) ifFalse: [ ^ nil ].
	link := PRLink fromString: ((self 
			parameterAt: 'to'
			ifAbsent: [ ^ nil ]) ifNil: [ ^ nil ]).
	link owner: aContext structure.
	link isBroken ifTrue: [ ^ nil ].
	url := link isInternal 
		ifTrue: [ (aContext structure: link target) url ]
		ifFalse: 
			[ link isExternal 
				ifTrue: [ link url ]
				ifFalse: [ ^ nil ] ].
	aContext session requestContext responseGenerator
		redirectTo: url greaseString;
		respond.
	^ nil! !

!PRValueLink methodsFor: '*pier-seaside' stamp: 'lr 5/21/2009 09:49'!
tocIn: aContext
	<value: 'toc' comment: 'Display the table of contents of the current structure.'>

	^ self lookupStructure: aContext structure do: [ :structure | [ :html |
		PRTocRenderer new 
			start: structure
			in: html painter
			on: html ] ]! !
PRViewRenderer initialize!
